@@ -31,7 +31,7 @@ from typing import Iterable, Generator, Union, Tuple, Any, overload, NewType, Di
 import numpy as np
 
 __all__ = ['get_session', 'set_session', 'get_session_value', 'is_autocast_enabled', 'set_autocast_enabled', 'get_backend', 'get_plateform', 'get_image_backend', 'get_trident_dir',
-           'epsilon', 'floatx', 'import_or_install',
+           'epsilon', 'floatx', 'import_or_install','compile_and_install_module',
            'check_keys', 'make_sure', 'if_none', 'camel2snake', 'snake2camel', 'to_onehot', 'to_list', 'addindent', 'format_time',
            'get_time_suffix', 'get_file_modified_time', 'get_function', 'get_class', 'get_terminal_size', 'gcd', 'get_divisors', 'isprime',
            'next_prime', 'prev_prime', 'nearest_prime', 'PrintException', 'TensorShape', 'unpack_singleton', 'enforce_singleton',
@@ -40,7 +40,7 @@ __all__ = ['get_session', 'set_session', 'get_session_value', 'is_autocast_enabl
            'remove_nonprintable',
 
            'GetImageMode', 'split_path', 'make_dir_if_need', 'sanitize_path', 'ShortcutMode', 'adaptive_format', 'num_cpus',
-           'get_args_spec', 'get_gpu_memory_map', 'get_memory_profile', 'get_gpu_memory_map', 'dtype', 'red_color', 'green_color', 'cyan_color', 'blue_color', 'orange_color',
+           'get_args_spec', 'get_gpu_memory_map', 'get_memory_profile', 'red_color', 'green_color', 'cyan_color', 'blue_color', 'orange_color',
            'gray_color', 'yellow_color','open_browser','launchTensorBoard','launchMLFlow']
 
 # In some cases, these basic types are shadowed by corresponding
@@ -50,6 +50,10 @@ __all__ = ['get_session', 'set_session', 'get_session_value', 'is_autocast_enabl
 import six
 from trident import context
 import warnings
+
+from math import e , nan , inf , pi
+__all__.extend(['e', 'pi', 'nan', 'inf'])
+
 warnings.simplefilter("ignore")
 _int = builtins.int
 _float = builtins.float
@@ -315,7 +319,7 @@ def adaptive_format(num: numbers.Number, prev_value: Union[numbers.Number, Itera
     elif name is not None and len(name) >= 3 and (name.endswith('s')):
         return '{0:{1}}'.format(num, '.3f')
     else:
-        format_string = ',.3'
+        format_string = ',.3f'
         if value_type == 'metric':
             if math.modf(num)[0] == 0:
                 num = int(num)
@@ -386,118 +390,117 @@ class DeviceType(object):
     type: str  # THPDevice_type
     index: _int  # THPDevice_index
 
-
-class dtype:
-    backend = get_backend()
-    if backend == 'pytorch':
-        import torch
-        # type definition
-        bool = torch.bool
-
-        int8 = torch.int8
-        byte = torch.int8
-        int16 = torch.int16
-        short = torch.int16
-        int32 = torch.int32
-        intc = torch.int32
-        int64 = torch.int64
-        intp = torch.int64
-
-        uint8 = torch.uint8
-        ubyte = torch.uint8
-        float16 = torch.float16
-        half = torch.float16
-        float32 = torch.float32
-        single = torch.float32
-        float64 = torch.float64
-        double = torch.float64
-        long = torch.int64
-        float = torch.float32
-        complex64 = torch.complex64
-        complex128 = torch.complex128
-        cfloat = torch.cfloat
-    elif backend == 'tensorflow':
-        import tensorflow as tf
-        bool = tf.bool
-
-        int8 = tf.int8
-        byte = tf.int8
-        int16 = tf.int16
-        short = tf.int16
-        int32 = tf.int32
-        intc = tf.int32
-        int64 = tf.int64
-        intp = tf.int64
-
-        uint8 = tf.uint8
-        ubyte = tf.uint8
-        float16 = tf.float16
-        half = tf.float16
-        float32 = tf.float32
-        single = tf.float32
-        float64 = tf.float64
-        double = tf.float64
-        long = tf.int64
-        float = tf.float32
-        complex64 = tf.complex64
-        complex128 = tf.complex128
-        cfloat = tf.complex64
-    elif backend == 'onnx':
-        from onnx import onnx_pb
-        bool = onnx_pb.TensorProto.BOOL
-        int8 = onnx_pb.TensorProto.INT8
-        byte = onnx_pb.TensorProto.INT8
-        int16 = onnx_pb.TensorProto.INT16
-        short = onnx_pb.TensorProto.INT16
-        int32 = onnx_pb.TensorProto.INT32
-        intc = onnx_pb.TensorProto.INT32
-        int64 = onnx_pb.TensorProto.INT64
-        intp = onnx_pb.TensorProto.INT64
-
-        uint8 = onnx_pb.TensorProto.UINT8
-        ubyte = onnx_pb.TensorProto.UINT8
-        float16 = onnx_pb.TensorProto.FLOAT1
-        half = onnx_pb.TensorProto.FLOAT1
-        float32 = onnx_pb.TensorProto.FLOAT
-        single = onnx_pb.TensorProto.FLOAT
-        float64 = onnx_pb.TensorProto.DOUBLE
-        double = onnx_pb.TensorProto.DOUBLE
-        long = onnx_pb.TensorProto.INT64
-        float = onnx_pb.TensorProto.FLOAT
-    elif backend == 'numpy':
-        bool = np.bool
-
-        int8 = np.int8
-        byte = np.int8
-        int16 = np.int16
-        short = np.int16
-        int32 = np.int32
-        intc = np.int32
-        int64 = np.int64
-        intp = np.int64
-
-        uint8 = np.uint8
-        ubyte = np.uint8
-        float16 = np.float16
-        half = np.float16
-        float32 = np.float32
-        single = np.float32
-        float64 = np.float64
-        double = np.float64
-        long = np.int64
-        float = np.float32
-        complex64 = np.complex64
-        complex128 = np.complex128
-        cfloat = np.complex64
-
-    def __repr__(self):
-        return 'dtype'
+#
+# class dtype:
+#     backend = get_backend()
+#     if backend == 'pytorch':
+#         import torch
+#         # type definition
+#         bool = torch.bool
+#         int8 = torch.int8
+#         byte = torch.int8
+#         int16 = torch.int16
+#         short = torch.int16
+#         int32 = torch.int32
+#         intc = torch.int32
+#         int64 = torch.int64
+#         intp = torch.int64
+#
+#         uint8 = torch.uint8
+#         ubyte = torch.uint8
+#         float16 = torch.float16
+#         half = torch.float16
+#         float32 = torch.float32
+#         single = torch.float32
+#         float64 = torch.float64
+#         double = torch.float64
+#         long = torch.int64
+#         float = torch.float32
+#         complex64 = torch.complex64
+#         complex128 = torch.complex128
+#         cfloat = torch.cfloat
+#     elif backend == 'tensorflow':
+#         import tensorflow as tf
+#         bool = tf.bool
+#         int8 = tf.int8
+#         byte = tf.int8
+#         int16 = tf.int16
+#         short = tf.int16
+#         int32 = tf.int32
+#         intc = tf.int32
+#         int64 = tf.int64
+#         intp = tf.int64
+#
+#         uint8 = tf.uint8
+#         ubyte = tf.uint8
+#         float16 = tf.float16
+#         half = tf.float16
+#         float32 = tf.float32
+#         single = tf.float32
+#         float64 = tf.float64
+#         double = tf.float64
+#         long = tf.int64
+#         float = tf.float32
+#         complex64 = tf.complex64
+#         complex128 = tf.complex128
+#         cfloat = tf.complex64
+#     elif backend == 'onnx':
+#         from onnx import onnx_pb
+#         bool = onnx_pb.TensorProto.BOOL
+#         int8 = onnx_pb.TensorProto.INT8
+#         byte = onnx_pb.TensorProto.INT8
+#         int16 = onnx_pb.TensorProto.INT16
+#         short = onnx_pb.TensorProto.INT16
+#         int32 = onnx_pb.TensorProto.INT32
+#         intc = onnx_pb.TensorProto.INT32
+#         int64 = onnx_pb.TensorProto.INT64
+#         intp = onnx_pb.TensorProto.INT64
+#
+#         uint8 = onnx_pb.TensorProto.UINT8
+#         ubyte = onnx_pb.TensorProto.UINT8
+#         float16 = onnx_pb.TensorProto.FLOAT1
+#         half = onnx_pb.TensorProto.FLOAT1
+#         float32 = onnx_pb.TensorProto.FLOAT
+#         single = onnx_pb.TensorProto.FLOAT
+#         float64 = onnx_pb.TensorProto.DOUBLE
+#         double = onnx_pb.TensorProto.DOUBLE
+#         long = onnx_pb.TensorProto.INT64
+#         float = onnx_pb.TensorProto.FLOAT
+#     elif backend == 'numpy':
+#         bool = np.bool
+#
+#         int8 = np.int8
+#         byte = np.int8
+#         int16 = np.int16
+#         short = np.int16
+#         int32 = np.int32
+#         intc = np.int32
+#         int64 = np.int64
+#         intp = np.int64
+#
+#         uint8 = np.uint8
+#         ubyte = np.uint8
+#         float16 = np.float16
+#         half = np.float16
+#         float32 = np.float32
+#         single = np.float32
+#         float64 = np.float64
+#         double = np.float64
+#         long = np.int64
+#         float = np.float32
+#         complex64 = np.complex64
+#         complex128 = np.complex128
+#         cfloat = np.complex64
+#
+#     def __repr__(self):
+#         return 'dtype'
 
 
 # class Device(object):
 #     '''
 #     Describes device type and device id
-#     syntax: device_type:device_id(optional)
+#     syntax: device_type:device_id(option
+#     al)
 #     example: 'CPU', 'CUDA', 'CUDA:1'
 #     '''
 #
@@ -759,12 +762,12 @@ class TensorShape(object):
                     return False
         return True
 
-    def get_dummy_tensor(self):
+    def get_dummy_tensor(self,batch_size=2):
         shape = [d for d in self._dims]
         if shape[0] is None:
-            shape[0] = 2
+            shape[0] = batch_size
         else:
-            shape = [2, ] + shape
+            shape = [batch_size, ] + shape
         return np.clip(np.abs(np.random.standard_normal(shape)), 0, 1)
 
 
@@ -899,6 +902,21 @@ class AverageMeter(object):
         self.count += n
         self.avg = self.sum / self.count
 
+def compile_and_install_module(module_name: str, source_code: str) -> types.ModuleType:
+    """Compile source code and install it as a module.
+
+    End result is that `import <module_name>` and `from <module_name> import ...` should work.
+    """
+    module = types.ModuleType(module_name, "Module created from source code")
+
+    # Execute source in context of empty/fake module
+    exec(source_code, module.__dict__)
+
+    # Insert fake module into sys.modules. It's now a real module
+    sys.modules[module_name] = module
+
+    # Imports should work now
+    return import_module(module_name)
 
 def import_or_install(package_name: str, install_package_name: str = None) -> None:
     """Import [package_name] if possibile, or install [install_package_name] and then import it
@@ -920,9 +938,9 @@ def import_or_install(package_name: str, install_package_name: str = None) -> No
     except ImportError:
         if install_package_name is None:
             install_package_name = package_name
-        if os.system('PIP --version') == 0:
+        if os.system('PIP --version') ==0:
             # No error from running PIP in the Command Window, therefor PIP.exe is in the %PATH%
-            os.system('PIP install {0}'.format(install_package_name))
+            os.system('PIP install {0}  --upgrade'.format(install_package_name))
             importlib.import_module(package_name)
         else:
             # Error, PIP.exe is NOT in the Path!! So I'll try to find it.
@@ -930,14 +948,15 @@ def import_or_install(package_name: str, install_package_name: str = None) -> No
             pip_location_attempt_2 = sys.executable.replace("python.exe", "") + "scripts\pip.exe"
             if os.path.exists(pip_location_attempt_1):
                 # The Attempt #1 File exists!!!
-                os.system(pip_location_attempt_1 + " install " + install_package_name)
+                os.system(pip_location_attempt_1 + " install " + install_package_name+'  --upgrade')
                 importlib.import_module(package_name)
             elif os.path.exists(pip_location_attempt_2):
                 # The Attempt #2 File exists!!!
-                os.system(pip_location_attempt_2 + " install " + install_package_name)
+                os.system(pip_location_attempt_2 + " install " + install_package_name+'  --upgrade')
                 importlib.import_module(package_name)
             else:
                 # Neither Attempts found the PIP.exe file, So i Fail...
+                print('Neither Attempts found the PIP.exe file, So i Fail...')
                 exit()
 
 
@@ -1627,6 +1646,42 @@ def num_cpus():
         return os.cpu_count()
 
 
+
+def get_gpu_memory_map():
+    """Get the current gpu usage.
+
+    Returns
+    -------
+    usage: dict
+        Keys are device ids as integers.
+        Values are memory usage as integers in MB.
+    """
+    result = subprocess.check_output(
+        [
+            'nvidia-smi', '--query-gpu=timestamp,index,name,utilization.gpu,memory.used,memory.free,memory.total,temperature.gpu',
+            '--format=csv,nounits'
+        ], encoding='utf-8')
+    gpu_memory = [x.split(',') for x in result.strip().split('\n')]
+
+    header=[h.strip() for h in gpu_memory[0]]
+    memory_map_list=[]
+    for n in range(1,len(gpu_memory)):
+        memory_map = OrderedDict()
+        for i,(k,v) in enumerate(zip(header,gpu_memory[n])):
+            if i==0:
+                memory_map[k]=datetime.datetime.strptime(v, "%Y/%m/%d %H:%M:%S.%f")
+            elif i == 1:
+                memory_map[k] =int(v)
+            elif i == 3:
+                memory_map[k] =float(v)
+            elif i > 3:
+                memory_map[k] = float(v)
+            else:
+                memory_map[k] = v
+        memory_map['memory usage'] =memory_map[memory_map.key_list[-4]]/memory_map[memory_map.key_list[-2]]
+        memory_map_list.append(memory_map)
+    return memory_map_list
+
 def get_memory_profile(mode):
     """
     'all' means return memory for all gpus
@@ -1654,25 +1709,6 @@ def get_memory_profile(mode):
     return memory_map
 
 
-def get_gpu_memory_map():
-    """Get the current gpu usage.
-
-    Returns
-    -------
-    usage: dict
-        Keys are device ids as integers.
-        Values are memory usage as integers in MB.
-    """
-    result = subprocess.check_output(
-        [
-            'nvidia-smi', '--query-gpu=memory.used',
-            '--format=csv,nounits,noheader'
-        ], encoding='utf-8')
-    gpu_memory = [int(x) for x in result.strip().split('\n')]
-    gpu_memory_map = dict(zip(range(len(gpu_memory)), gpu_memory))
-    return gpu_memory_map
-
-
 def map_function_arguments(params, params_dict, *args, **kwargs):
     """
     Helper to determine the argument map for use with various call operations.
