@@ -40,6 +40,9 @@ if get_backend() == 'pytorch':
     double = torch.float64
     long = torch.int64
     float= torch.float32
+    complex64 = torch.complex64
+    complex128 = torch.complex128
+    cfloat = torch.cfloat
 
 
 elif get_backend() == 'tensorflow':
@@ -65,7 +68,34 @@ elif get_backend() == 'tensorflow':
     double = tf.float64
     long=tf.int64
     float =  tf.float32
+    complex64 = tf.complex64
+    complex128 = tf.complex128
+    cfloat = tf.complex64
+elif get_backend() == 'onnx':
+    from onnx import onnx_pb
+    bool = onnx_pb.TensorProto.BOOL
+    int8 = onnx_pb.TensorProto.INT8
+    byte = onnx_pb.TensorProto.INT8
+    int16 = onnx_pb.TensorProto.INT16
+    short = onnx_pb.TensorProto.INT16
+    int32 = onnx_pb.TensorProto.INT32
+    intc = onnx_pb.TensorProto.INT32
+    int64 = onnx_pb.TensorProto.INT64
+    intp = onnx_pb.TensorProto.INT64
 
+    uint8 = onnx_pb.TensorProto.UINT8
+    ubyte = onnx_pb.TensorProto.UINT8
+    float16 = onnx_pb.TensorProto.FLOAT1
+    half = onnx_pb.TensorProto.FLOAT1
+    float32 = onnx_pb.TensorProto.FLOAT
+    single = onnx_pb.TensorProto.FLOAT
+    float64 = onnx_pb.TensorProto.DOUBLE
+    double = onnx_pb.TensorProto.DOUBLE
+    long = onnx_pb.TensorProto.INT64
+    float = onnx_pb.TensorProto.FLOAT
+    complex64 = None
+    complex128  =None
+    cfloat  =None
 else:
     bool = np.bool
 
@@ -88,4 +118,7 @@ else:
     double = np.float64
     long = np.int64
     float= np.float32
+    complex64 = np.complex64
+    complex128 = np.complex128
+    cfloat = np.complex64
 
