@@ -19,16 +19,17 @@ import numpy as np
 from trident.callbacks.callback_base import LambdaCallback, UnfreezeModelCallback
 
 from trident.backend.common import to_list, format_time, get_terminal_size, get_session, get_backend, \
-    PrintException, OrderedDict, split_path, sanitize_path, adaptive_format, dtype, cyan_color, get_class
+    PrintException, OrderedDict, split_path, sanitize_path, adaptive_format,  cyan_color, get_class,camel2snake
+from trident.backend import dtype
 from trident.backend.opencv_backend import array2image
 from trident.backend.tensorspec import *
-
+from trident import context
 from trident.data.image_common import *
 from trident.data.vision_transforms import Unnormalize
 from trident.loggers.history import HistoryBase
 
-_session = get_session()
-working_directory = _session.working_directory
+ctx = context._context()
+working_directory = ctx.working_directory
 
 if get_backend() == 'pytorch':
     from trident.backend.pytorch_backend import *
@@ -87,6 +88,7 @@ class ModelBase(object):
             'epoch_losses': HistoryBase('epoch_losses'),
             'epoch_metrics': HistoryBase('epoch_metrics'),
             'grads_state': OrderedDict(),
+            'tmp_losses': HistoryBase('tmp_losses'),
             'tmp_metrics': HistoryBase('tmp_metrics'),
             'out_sample_metrics': HistoryBase('out_sample_metrics'),
             'print_progress_frequency': 10,
@@ -114,8 +116,11 @@ class ModelBase(object):
             'best_model': None,  # current model
             'loss_history': None, 'metric_history': None, 'base_lr': None,  # current loss
             'current_lr': None,  # current loss
-            'save_path': os.path.join(working_directory, 'Models'), 'is_collect_data': True, 'callbacks': [],
-            'stop_update': 0, 'retain_graph': False,
+            'save_path': os.path.join(working_directory, 'Models'),
+            'is_collect_data': True,
+            'callbacks': [],
+            'stop_update': 0,
+            'retain_graph': False,
             'skip_generate_output': False,
             'skip_reset_total_loss': False}
         if name is not None:
@@ -283,7 +288,7 @@ class ModelBase(object):
                 self._outputs[arg_names[i + len(old_inputs)]] = spec
                 self._targets[target_arg] = spec
 
-            print(self._signature)
+            ctx.print(self._signature)
         elif not isinstance(arg_names, (list, tuple)):
             raise ValueError('arg_names should be list or tuple')
         elif len(self._signature) != len(arg_names):
@@ -346,7 +351,7 @@ class ModelBase(object):
     def _initial_graph(self, inputs=None, input_shape=None, output=None):
         pass
 
-    def complie(self, optimizer="Ranger",
+    def complie(self, optimizer="AdaBelief",
                 loss=None,
                 metrics=None,
                 loss_weights=None,
@@ -377,8 +382,31 @@ class ModelBase(object):
             ):
         raise NotImplementedError
 
+
+    def fit_generator(self,
+            generator,
+            batch_size=None,
+            epochs=1,
+            verbose=1,
+            callbacks=None,
+            validation_split=0.0,
+            validation_data=None,
+            shuffle=True,
+            class_weight=None,
+            sample_weight=None,
+            initial_epoch=0,
+            steps_per_epoch=None,
+            validation_steps=None,
+            validation_batch_size=None,
+            validation_freq=1,
+            max_queue_size=10,
+            workers=1,
+            use_multiprocessing=False,
+            ):
+        raise NotImplementedError
+
     def __getattr__(self, name):
-        if name in ['_input_shape', '_output_shape', '_class_names', 'class_names', 'output_fn', 'optimizer']:
+        if name in ['training_context','_input_shape', '_output_shape', '_class_names', 'class_names', 'output_fn', 'optimizer']:
             return self.__dict__[name]
         elif name in ['reverse_preprocess_flow']:
             return self.__getattribute__(name)
@@ -390,7 +418,7 @@ class ModelBase(object):
                 elif is_tensor(_model):
                     object.__setattr__(self, name, 'model_' + str(uuid.uuid4().node))
                     return self.__dict__[name]
-        if name == 'signature' or name == '_signature':
+        elif name == 'signature' or name == '_signature':
             _model = self.__dict__['_model']
             if _model is not None and hasattr(_model, '_signature'):
                 return _model._signature
@@ -477,7 +505,7 @@ class ModelBase(object):
     def with_optimizer(self, optimizer, **kwargs):
         return self
 
-    def with_loss(self, loss, loss_weight=1, start_epoch=0, name='', **kwargs):
+    def with_loss(self, loss, loss_weight=1, start_epoch=0,as_metric=False, name='', **kwargs):
         return self
 
     def with_metric(self, metric, name='', print_only=False, **kwargs):
@@ -546,6 +574,7 @@ class ModelBase(object):
             'epoch_losses': HistoryBase('epoch_losses'),
             'epoch_metrics': HistoryBase('epoch_metrics'),
             'grads_state': OrderedDict(),
+            'tmp_losses': HistoryBase('tmp_losses'),
             'tmp_metrics': HistoryBase('tmp_metrics'),
             'out_sample_metrics': HistoryBase('out_sample_metrics'),
             'print_progress_frequency': 10,
@@ -648,8 +677,11 @@ class ModelBase(object):
                 model.train()
 
             signature = model.signature
-
+            # if any_abnormal_number(data['image']):
+            #     print('{0} before calculate forward'.format(''))
             output = try_map_args_and_call(model, data, data_feed, self.is_autocast_enabled)
+            # if any_abnormal_number(output):
+            #     print('{0} after calculate forward'.format(''))
             if isinstance(output, (list, tuple)):
                 for i in range(len(output)):
                     data[signature.outputs.key_list[i]] = output[i]
@@ -673,13 +705,12 @@ class ModelBase(object):
             callback.on_loss_calculation_start(self.training_context)
 
     def do_on_loss_calculation_end(self):
+
         for callback in self.callbacks:
             callback.on_loss_calculation_end(self.training_context)
 
     def do_calculate_losses(self):
         self.do_on_loss_calculation_start()
-        # confirm singleton
-        # output=unpack_singleton(output)
 
         # losss
         for k, v in self._losses.items():
@@ -690,6 +721,8 @@ class ModelBase(object):
                     if k in self.loss_weights:
                         loss_weight = self.loss_weights[k]
                     loss_weight = to_tensor(loss_weight, 'float32')
+                    #print(-1, 'train_data', self.train_data.value_list[-1].shape, 'abnormal:', any_abnormal_number( self.train_data.value_list[-1]))
+                    #print(self.train_data.value_list[-1])
                     this_loss = loss_weight * try_map_args_and_call(v, self.train_data, self.training_context['data_feed'],
                                                                     self.is_autocast_enabled)  # v.forward(output, target) if hasattr(v, 'forward') else v(
 
@@ -703,8 +736,11 @@ class ModelBase(object):
                                 overall_loss = overall_loss + this_loss[i]
                         self.training_context['current_loss'] = self.training_context['current_loss'] + overall_loss
 
-                        if self.training_context['is_collect_data']:
-                            self.training_context['losses'].collect(k, self.training_context['steps'], overall_loss)
+                        if hasattr(v, 'as_metric') and v.as_metric == True:
+                            self.training_context['tmp_metrics'].collect(camel2snake(k), self.training_context['steps'],to_numpy(overall_loss))
+
+
+                        self.training_context['tmp_losses'].collect(k, self.training_context['steps'], overall_loss)
 
                     else:
                         if any_abnormal_number(this_loss):
@@ -712,10 +748,18 @@ class ModelBase(object):
                         else:
                             # a leaf Variable that requires grad connotused in an in-place operation.
                             self.training_context['current_loss'] = self.training_context['current_loss'] + this_loss
-                        if self.training_context['is_collect_data']:
-                            self.training_context['losses'].collect(k, self.training_context['steps'], this_loss)
+
+                        if hasattr(v,'as_metric') and v.as_metric==True:
+                            self.training_context['tmp_metrics'].collect(camel2snake(k), self.training_context['steps'],to_numpy(this_loss))
+
+
+                        self.training_context['tmp_losses'].collect(k, self.training_context['steps'], this_loss)
+
+
+
+
                 except Exception as e:
-                    print(e)
+                    ctx.print(e)
                     PrintException()
 
         self.do_on_loss_calculation_end()
@@ -734,8 +778,8 @@ class ModelBase(object):
                 # a leaf Variable that requires grad connotused in an in-place operation.
                 self.training_context['current_loss'] = self.training_context['current_loss'] + this_loss  # self.training_context[
             # 'current_loss'] + this_loss
-            if self.training_context['is_collect_data']:
-                self.training_context['losses'].collect(k + '_Loss', self.training_context['steps'], this_loss)
+
+            self.training_context['tmp_losses'].collect(k + '_Loss', self.training_context['steps'], this_loss)
 
     def do_calculate_constraints(self):
         for k, v in self._constraints.items():
@@ -864,7 +908,7 @@ class ModelBase(object):
             else:
                 loss_value = to_numpy(loss_values).astype(np.float32).mean()
         step_time = self.training_context['time_batch_progress'] / builtins.min(self.steps + 1, print_batch_progress_frequency)
-        progress_bar(step_time, self.training_context['current_batch'], self.training_context['total_batch'] if self.training_context['total_batch'] is not None else '*',
+        progress_bar(step_time, self.training_context['current_batch']+1, self.training_context['total_batch'] if self.training_context['total_batch'] is not None else '*',
                      'Loss: {0} | {1} | lr: {2:<10.3e} | epoch: {3}'.format(adaptive_format(loss_value, value_type='loss'), ', '.join(metric_strings),
                                                                             self.training_context['current_lr'],
                                                                             self.training_context['current_epoch']),
@@ -935,13 +979,14 @@ class ModelBase(object):
             self.batch_loss_history.collect('epoch', self.training_context['steps'], current_epoch)
             self.batch_metric_history.collect('epoch', self.training_context['steps'], current_epoch)
             is_epoch_end = (self.training_context['current_batch'] == self.training_context['total_batch'] - 1) if self.training_context['total_batch'] is not None else done
-
+            # print('step:{0}'.format(self.training_context['steps']))
             self.do_on_batch_start()
             for callback in self.callbacks:
                 callback.on_batch_start(self.training_context)
 
             train_data, test_data = self.do_on_data_received(train_data, test_data)
-
+            # if any_abnormal_number(train_data.value_list[0]):
+            #     print('{0} after data_received'.format(''))
             for callback in self.callbacks:
                 callback.on_data_received(self.training_context)
 
@@ -956,11 +1001,14 @@ class ModelBase(object):
                         self.do_calculate_forward(is_training=True)
 
                 except Exception as e:
-                    print(e)
+                    ctx.print(e)
                     PrintException()
 
             # write output in to data
+            #print(-1, 'output',self.training_context['train_data'].value_list[-1].shape, 'abnormal:', any_abnormal_number(self.training_context['train_data'].value_list[-1]))
 
+            # if any_abnormal_number(train_data['output']):
+            #     print('{0} before calculate loss'.format(''))
             self.do_calculate_losses()
             self.do_calculate_regularizations()
 
@@ -993,8 +1041,7 @@ class ModelBase(object):
 
                     # check
                     if len(steps) > 0:
-                        values = to_scalar(values)
-                        self.training_context['metrics'].collect(k, self.training_context['steps'], values)
+                        self.training_context['metrics'].collect(k, self.training_context['steps'], to_scalar(to_numpy(values).mean()))
                 self.training_context['tmp_metrics'].reset()
 
             # ON_BATCH_END
@@ -1019,7 +1066,7 @@ class ModelBase(object):
                     history_metric_value = to_scalar(test_values[-1])
                     verbose.append('{0}: {1}'.format(k, adaptive_format(history_metric_value,prev_value=test_values,value_type='metric', name=k)))
                 out_sample_evaluation_str = cyan_color(self.training_context['model_name'] + ' ' + 'out-of-sample evaluation: ' + ','.join(verbose))
-                print(out_sample_evaluation_str)
+                ctx.print(out_sample_evaluation_str)
             # self.training_context['steps'] += 1
 
         except KeyboardInterrupt as k:
@@ -1065,6 +1112,10 @@ class ModelBase(object):
     def gpu(self):
         self.cuda()
 
+
+    def xpu(self):
+        self._model.xpu()
+
     def train(self, mode=True):
         if isinstance(self._model, Layer):
             self._model.train(mode)
@@ -1086,7 +1137,7 @@ def progress_bar(step_time, current, total, msg=None, name=''):
     # cur_len = builtins.max(int(TOTAL_BAR_LENGTH * float(current) / total), 1)
     # rest_len = int(TOTAL_BAR_LENGTH - cur_len) - 1 + cur_len
 
-    L = ['{0}'.format(name) + '   ', ' Step: {0:<8s}'.format(format_time(step_time))]
+    L = ['{0}'.format(name) , ' Step: {0:<8s}'.format(format_time(step_time))]
     # L.append(' | Tot: {0:<12s}'.format(format_time(tot_time)))
     if msg:
         L.append(' | ' + msg)
