@@ -20,7 +20,7 @@ from trident.data.image_common import image_backend_adaption
 from trident.data.mask_common import label2color
 from trident.misc.ipython_utils import is_in_ipython, is_in_colab
 from trident.misc.visualization_utils import *
-
+from trident.optims.losses import _check_logsoftmax_logit
 if get_backend() == 'pytorch':
     from trident.backend.pytorch_backend import try_map_args_and_call, Layer,Sequential
     from trident.backend.pytorch_ops import to_numpy, arange, cast, clip, sqrt, int_shape, argmax, softmax, ndim, exp, \
@@ -37,7 +37,7 @@ ctx = context._context()
 _backend = get_backend()
 
 __all__ = ['VisualizationCallbackBase', 'TileImageCallback', 'PrintGradientsCallback', 'SegTileImageCallback',
-           'PlotLossMetricsCallback', 'DetectionPlotImageCallback', 'GanTileImageCallback']
+           'PlotLossMetricsCallback', 'DetectionPlotImageCallback', 'GanTileImageCallback','PrintGpuUtilizationCallback']
 
 
 class VisualizationCallbackBase(CallbackBase):
@@ -51,7 +51,7 @@ class VisualizationCallbackBase(CallbackBase):
         else:
             print(red_color('Only [batch, step, epoch] are valid unit.', True))
         if save_path is None:
-            save_path = 'results'
+            save_path = 'Results'
         self.save_path = make_dir_if_need(save_path)
         self.imshow = imshow
         if imshow is None and self.is_in_ipython:
@@ -61,7 +61,7 @@ class VisualizationCallbackBase(CallbackBase):
 
 
 class TileImageCallback(VisualizationCallbackBase):
-    def __init__(self, frequency=-1, unit='batch', save_path: str = 'results',
+    def __init__(self, frequency=-1, unit='batch', save_path: str = 'Results',
                  name_prefix: str = 'tile_image_{0}.png', row=3, include_input=True, include_output=True, include_target=True,
                  include_mask=None, reverse_image_transform=None, imshow=False):
         super(TileImageCallback, self).__init__(frequency, unit, save_path, imshow)
@@ -69,7 +69,6 @@ class TileImageCallback(VisualizationCallbackBase):
         self.is_in_colab = is_in_colab()
         self.tile_image_name_prefix = name_prefix
         self.reverse_image_transform = reverse_image_transform
-
         self.row = row
 
         self.include_input = include_input
@@ -89,26 +88,27 @@ class TileImageCallback(VisualizationCallbackBase):
         data_feed = training_context['data_feed']
         data = training_context['train_data']
         model = training_context['current_model']
-        dataprovider = enforce_singleton(ctx.get_data_provider())
+        dataprovider =ctx.get_data_provider()[-1]
         if self.reverse_image_transform is None:
-            if len(ctx.get_data_provider()) == 1:
-                self.reverse_image_transform = dataprovider.reverse_image_transform
-                #self.reverse_image_transform_funcs = dataprovider.reverse_image_transform_funcs
+            self.reverse_image_transform = dataprovider.reverse_image_transform
+            #self.reverse_image_transform_funcs = dataprovider.reverse_image_transform_funcs
+
 
         if self.include_input:
             input = to_numpy(data[data_feed[model.signature.inputs.key_list[0]]])
+
         if 'tensor' in model.__class__.__name__.lower():
-            output = to_numpy(model.clone())
+            output = to_numpy(model)
         elif isinstance(model, Layer):
-            output = to_numpy(data[data_feed[model.signature.outputs.key_list[0]]].copy())
+            output = to_numpy(data[data_feed[model.signature.outputs.key_list[0]]])
         if self.include_target:
             candidate = [k for k in data_feed.keys() if 'target' in k]
             if len(candidate) == 1:
-                target = to_numpy(data[data_feed[candidate[0]]].copy())
+                target = to_numpy(data[data_feed[candidate[0]]])
             elif dataprovider.traindata.label is not None and not isinstance(dataprovider.traindata.label, ZipDataset):
-                target = to_numpy(data[data_feed[dataprovider.traindata.label.symbol]].copy())
+                target = to_numpy(data[data_feed[dataprovider.traindata.label.symbol]])
             elif dataprovider.traindata.label is not None and isinstance(dataprovider.traindata.label, ZipDataset):
-                target = to_numpy(data[data_feed[dataprovider.traindata.label.items[0].symbol]].copy())
+                target = to_numpy(data[data_feed[dataprovider.traindata.label.items[0].symbol]])
 
         if self.include_mask:
             if isinstance(dataprovider.traindata.label, MaskDataset):
@@ -179,10 +179,12 @@ class TileImageCallback(VisualizationCallbackBase):
         fig = tile_rgb_images(*tile_images_list, row=self.row, save_path=os.path.join(self.save_path, self.tile_image_name_prefix), imshow=True, legend=legend)
         if ctx.enable_tensorboard and ctx.summary_writer is not None:
             ctx.summary_writer.add_figure(training_context['training_name'] + '/plot/tile_image', fig, global_step=training_context['steps'], close=True, walltime=time.time())
+        if ctx.enable_mlflow and ctx.mlflow_logger is not None:
+            ctx.mlflow_logger.add_figure(tag=training_context['training_name'] + '/plot/tile_image', fig=fig, artifact_file=self.save_path)
         plt.close()
 
     def on_batch_end(self, training_context):
-        if self.frequency > 0 and ((self.unit == 'batch' and (training_context['current_batch'] + 1) % self.frequency == 0) or (
+        if self.frequency > 0 and ((self.unit == 'batch' and (training_context['steps'] + 1) % self.frequency == 0) or (
                 self.unit == 'step' and (training_context['steps'] + 1) % self.frequency == 0)):
             self.plot_tile_image(training_context)
 
@@ -192,7 +194,7 @@ class TileImageCallback(VisualizationCallbackBase):
 
 
 class GanTileImageCallback(VisualizationCallbackBase):
-    def __init__(self, frequency=-1, unit='batch', save_path: str = 'results',
+    def __init__(self, frequency=-1, unit='batch', save_path: str = 'Results',
                  name_prefix: str = 'tile_image_{0}.png', row=3,
                  include_mask=None, reverse_image_transform=None, imshow=False):
         super(GanTileImageCallback, self).__init__(frequency, unit, save_path, imshow)
@@ -217,14 +219,18 @@ class GanTileImageCallback(VisualizationCallbackBase):
         data_feed = training_context['data_feed']
         data = training_context['train_data']
         model = training_context['current_model'].eval()
-        dataprovider = enforce_singleton(ctx.get_data_provider())
+
+        dataprovider =ctx.get_data_provider()[-1]
+        if self.reverse_image_transform is None:
+            self.reverse_image_transform = dataprovider.reverse_image_transform
+
         output = to_numpy(model(data[data_feed['input']])) if data_feed['input'] in data else to_numpy(data['output'])
         model.train()
-        reverse_image_transform = dataprovider.reverse_image_transform
 
-        if reverse_image_transform is not None:
+
+        if self.reverse_image_transform is not None:
             for i in range(len(output)):
-                self.output_arr.append(reverse_image_transform(output[i]))
+                self.output_arr.append(self.reverse_image_transform(output[i]))
                 if len(self.output_arr) == self.row:
                     self.tile_images_list.append(self.output_arr)
                     if len(self.tile_images_list) == self.row:
@@ -241,7 +247,7 @@ class GanTileImageCallback(VisualizationCallbackBase):
             plt.close()
 
     def on_batch_end(self, training_context):
-        if self.frequency > 0 and ((self.unit == 'batch' and (training_context['current_batch'] + 1) % self.frequency == 0) or (
+        if self.frequency > 0 and ((self.unit == 'batch' and (training_context['steps'] + 1) % self.frequency == 0) or (
                 self.unit == 'step' and (training_context['steps'] + 1) % self.frequency == 0) or not self.sample_enough):
             if self.sample_enough:
                 self.tile_images_list = []
@@ -260,12 +266,12 @@ class GanTileImageCallback(VisualizationCallbackBase):
 
 
 class SegTileImageCallback(VisualizationCallbackBase):
-    def __init__(self, frequency=-1, unit='batch', save_path: str = 'results', reverse_image_transform=None,
-                 is_label_mask=False, palette=None, background=(120, 120, 120), name_prefix: str = 'segtile_image_{0}.png', imshow=False):
+    def __init__(self, frequency=-1, unit='batch', save_path: str = 'Results', reverse_image_transform=None,
+                  palette=None, background=(120, 120, 120), name_prefix: str = 'segtile_image_{0}.png', imshow=False,**kwargs):
         super(SegTileImageCallback, self).__init__(frequency, unit, save_path, imshow)
         self.is_in_ipython = is_in_ipython()
         self.is_in_colab = is_in_colab()
-        self.is_label_mask = is_label_mask
+
         self.palette = palette
         self.tile_image_name_prefix = name_prefix
         self.reverse_image_transform = reverse_image_transform
@@ -280,18 +286,17 @@ class SegTileImageCallback(VisualizationCallbackBase):
         new_shape[axis] = 3
         background = np.reshape(to_numpy(self.background), new_shape)
         tile_images_list = []
-        input = None
+
         mask = None
         output = None
-        is_label_mask = self.is_label_mask
+
         data_feed = training_context['data_feed']
         data = training_context['train_data']
         model = training_context['current_model']
-        dataprovider = enforce_singleton(ctx.get_data_provider())
+        dataprovider = ctx.get_data_provider()[-1]
         if self.reverse_image_transform is None:
-            if len(ctx.get_data_provider()) == 1:
-                self.reverse_image_transform = dataprovider.reverse_image_transform
-                self.reverse_image_transform_funcs = dataprovider.reverse_image_transform_funcs
+            self.reverse_image_transform = dataprovider.reverse_image_transform
+
 
         input = to_numpy(data[data_feed[model.signature.inputs.key_list[0]]])
         if 'tensor' in model.__class__.__name__.lower():
@@ -300,6 +305,8 @@ class SegTileImageCallback(VisualizationCallbackBase):
             output = to_numpy(data[data_feed[model.signature.outputs.key_list[0]]].copy())
         mask_type=None
         output_arr=output.copy()
+        if _check_logsoftmax_logit(output_arr):
+            output_arr=exp(output_arr)
         if isinstance(dataprovider.traindata.label, MaskDataset):
             mask_type=dataprovider.traindata.label.object_type
             mask = to_numpy(data[dataprovider.traindata.label.symbol])
@@ -308,37 +315,37 @@ class SegTileImageCallback(VisualizationCallbackBase):
                 if isinstance(ds, MaskDataset):
                     mask_type =ds.object_type
                     mask = to_numpy(data[ds.symbol])
-        if mask_type==ObjectType.label_mask:
-            mask = label2color(mask, self.palette)
-            if ndim(output)==4:
-                output=argmax(output,1)
-            output_arr = label2color(output, self.palette)
+        if mask_type==ObjectType.label_mask or mask_type==ObjectType.color_mask:
+            mask = label2color(mask, palette=self.palette).astype(np.uint8)
+            if ndim(output_arr)==4:
+                output_arr=argmax(output_arr,axis)
+            output_arr = label2color(output_arr, palette=self.palette).astype(np.uint8)
         elif  mask_type==ObjectType.binary_mask:
-            if ndim(output)==4:
-                output=argmax(output,1)
-            output_arr=output
+            if ndim(output_arr)==4:
+                output_arr=argmax(output_arr,1)
+
         elif mask_type == ObjectType.alpha_mask:
             if get_backend() == 'tensorflow':
-                output_arr = output[:, :, :, 1:2] * argmax(output, axis)
+                output_arr = output_arr[:, :, :, 1:2] * argmax(output_arr, axis)
             else:
-                output_arr = (output[:, 1:2, :, :] * argmax(output, axis)).transpose(0, 2, 3, 1)
-        if  ndim(output)==4 and int_shape(output)[1 if get_backend() == 'pytorch' else -1]>4:
-            output  = argmax(output, 1 if get_backend() == 'pytorch' else -1)
-            output_arr = output
+                output_arr = (output_arr[:, 1:2, :, :] * argmax(output_arr, axis)).transpose(0, 2, 3, 1)
+        # if  ndim(output_arr)==4 and int_shape(output_arr)[1 if get_backend() == 'pytorch' else -1]>4:
+        #     output_arr  = argmax(output_arr, 1 if get_backend() == 'pytorch' else -1)
+        #
 
 
 
         input_arr = []
         input = to_numpy(input)
         for i in range(len(input)):
-            input_arr.append(image_backend_adaption(self.reverse_image_transform(input[i])))
+            input_arr.append(self.reverse_image_transform(input[i]))
         input_arr = np.stack(input_arr, axis=0)
         # input_arr=np.asarray(input_arr)
         tile_images_list.append(input_arr)
 
-        if is_label_mask:
+        if mask_type==ObjectType.label_mask or mask_type==ObjectType.color_mask:
             tile_images_list.append(mask)
-            tile_images_list.append(output)
+            tile_images_list.append(output_arr)
         else:
             target_arr = mask
 
@@ -353,7 +360,7 @@ class SegTileImageCallback(VisualizationCallbackBase):
                 else:
                     output_arr = np.expand_dims(output_arr, axis=axis)
 
-            if 'alpha' not in data:
+            if 'alpha' not  in data:
                 target_arr[target_arr > 0] = 1
 
             tile_images_list.append(target_arr * input_arr + (1 - target_arr) * background)
@@ -369,7 +376,7 @@ class SegTileImageCallback(VisualizationCallbackBase):
         plt.close()
 
     def on_batch_end(self, training_context):
-        if self.frequency > 0 and ((self.unit == 'batch' and (training_context['current_batch'] + 1) % self.frequency == 0) or (
+        if self.frequency > 0 and ((self.unit == 'batch' and (training_context['steps'] + 1) % self.frequency == 0) or (
                 self.unit == 'step' and (training_context['steps'] + 1) % self.frequency == 0)):
             self.plot_tile_image(training_context)
 
@@ -379,7 +386,7 @@ class SegTileImageCallback(VisualizationCallbackBase):
 
 
 class DetectionPlotImageCallback(VisualizationCallbackBase):
-    def __init__(self, frequency=-1, unit='batch', save_path: str = 'results', reverse_image_transform=None, labels=None,
+    def __init__(self, frequency=-1, unit='batch', save_path: str = 'Results', reverse_image_transform=None, labels=None,
                  palette=None, background=(120, 120, 120), name_prefix: str = 'detection_plot_image_{0}.png', imshow=False):
         super(DetectionPlotImageCallback, self).__init__(frequency, unit, save_path, imshow)
         self.is_in_ipython = is_in_ipython()
@@ -395,6 +402,9 @@ class DetectionPlotImageCallback(VisualizationCallbackBase):
         input = None
         target = None
         output = None
+        dataprovider = ctx.get_data_provider()[-1]
+        if self.reverse_image_transform is None:
+            self.reverse_image_transform = dataprovider.reverse_image_transform
 
         data_feed = training_context['data_feed']
         data = training_context['train_data']
@@ -431,7 +441,7 @@ class DetectionPlotImageCallback(VisualizationCallbackBase):
         plt.close()
 
     def on_batch_end(self, training_context):
-        if self.frequency > 0 and ((self.unit == 'batch' and (training_context['current_batch'] + 1) % self.frequency == 0) or (
+        if self.frequency > 0 and ((self.unit == 'batch' and (training_context['steps'] + 1) % self.frequency == 0) or (
                 self.unit == 'step' and (training_context['steps'] + 1) % self.frequency == 0)):
             self.plot_detection_image(training_context)
 
@@ -441,7 +451,7 @@ class DetectionPlotImageCallback(VisualizationCallbackBase):
 
 
 class PlotLossMetricsCallback(VisualizationCallbackBase):
-    def __init__(self, frequency=-1, unit='batch', save_path: str = 'results', clean_ipython_output_frequency=5,
+    def __init__(self, frequency=-1, unit='batch', save_path: str = 'Results', clean_ipython_output_frequency=5,
                  name_prefix: str = 'loss_metric_curve_{0}.png', is_inplace=False, imshow=False):
         super(PlotLossMetricsCallback, self).__init__(frequency, unit, save_path, imshow)
         self.training_items = None
@@ -454,54 +464,58 @@ class PlotLossMetricsCallback(VisualizationCallbackBase):
         self.counter = 0
         self.clean_ipython_output_frequency = clean_ipython_output_frequency
 
+
     def on_training_start(self, training_context):
         if not self.is_inplace:
             self.training_items = training_context['training_items']
 
     def on_overall_batch_end(self, training_context):
-        if not self.is_inplace and training_context['steps'] > 10:
-            if self.frequency > 0 and ((self.unit == 'batch' and (training_context['steps'] + 1) % self.frequency == 0) or (
-                    self.unit == 'step' and (training_context['steps'] + 1) % self.frequency == 0)):
-                if is_in_ipython() and self.counter == self.clean_ipython_output_frequency:
-                    display.clear_output(wait=True)
-                    self.counter = 0
-                self.loss_history_list = []
-                self.metric_history_list = []
-                plotable_metric_names = OrderedDict()
-                for i in range(len(self.training_items.value_list)):
-                    trainitem = self.training_items.value_list[i]
-                    self.loss_history_list.append(trainitem.batch_loss_history)
-                    plotable_metric_names[i] = [k for k, v in trainitem._metrics.item_list if v.print_only == False]
-                    self.metric_history_list.append(trainitem.batch_metric_history)
-                self.counter += 1
-                fig = loss_metric_curve(self.loss_history_list, self.metric_history_list, metrics_names=plotable_metric_names,
-                                        legend=training_context['training_names'].value_list, calculate_base='batch',
-                                        max_iteration=None, save_path=os.path.join(self.save_path, self.name_prefix),
-                                        imshow=self.imshow)
+        if self.frequency > 0 and ((self.unit == 'batch' and (training_context['steps'] + 1) % self.frequency == 0) or (
+                self.unit == 'step' and (training_context['steps'] + 1) % self.frequency == 0)):
+            if is_in_ipython() and self.counter == self.clean_ipython_output_frequency:
+                display.clear_output(wait=True)
+                self.counter = 0
+            self.loss_history_list = []
+            self.metric_history_list = []
+            plotable_metric_names = OrderedDict()
+            for i in range(len(self.training_items.value_list)):
+                trainitem = self.training_items.value_list[i]
+                self.loss_history_list.append(trainitem.batch_loss_history)
+                plotable_metric_names[i] = [k for k, v in trainitem._metrics.item_list if v.print_only == False]
+                self.metric_history_list.append(trainitem.batch_metric_history)
+            self.counter += 1
+            fig = loss_metric_curve(self.loss_history_list, self.metric_history_list, metrics_names=plotable_metric_names,
+                                    legend=training_context['training_names'].value_list, calculate_base='batch',
+                                    max_iteration=None, save_path=os.path.join(self.save_path, self.name_prefix),
+                                    imshow=self.imshow)
 
-                if ctx.enable_tensorboard and ctx.summary_writer is not None:
-                    ctx.summary_writer.add_figure('overall/plot/loss_metric_curve', fig, global_step=training_context['steps'], close=True, walltime=time.time())
-                plt.close()
-            elif self.unit  == 'epoch' and (training_context['epochs']  + 1) % self.frequency == 0 and  self.training_items.value_list[0].training_context['current_batch'] +1==self.training_items.value_list[0].training_context['total_batch'] :
-                if is_in_ipython() and self.counter == self.clean_ipython_output_frequency:
-                    display.clear_output(wait=True)
-                    self.counter = 0
-                self.loss_history_list = []
-                self.metric_history_list = []
-                plotable_metric_names = OrderedDict()
-                for i in range(len(self.training_items.value_list)):
-                    trainitem = self.training_items.value_list[i]
-                    self.loss_history_list.append(trainitem.epoch_loss_history)
-                    plotable_metric_names[i] = [k for k, v in trainitem._metrics.item_list if v.print_only == False]
-                    self.metric_history_list.append(trainitem.epoch_metric_history)
-                self.counter += 1
-                fig = loss_metric_curve(self.loss_history_list, self.metric_history_list, metrics_names=plotable_metric_names,
-                                        legend=training_context['training_names'].value_list, calculate_base='epoch',
-                                        max_iteration=None, save_path=os.path.join(self.save_path, self.name_prefix),
-                                        imshow=self.imshow)
-                if ctx.enable_tensorboard and ctx.summary_writer is not None:
-                    ctx.summary_writer.add_figure('overall/plot/loss_metric_curve', fig, global_step=training_context['steps'], close=True, walltime=time.time())
-                plt.close()
+            if ctx.enable_tensorboard and ctx.summary_writer is not None:
+                ctx.summary_writer.add_figure('overall/plot/loss_metric_curve', fig, global_step=training_context['steps'], close=True, walltime=time.time())
+            plt.close()
+
+
+    def on_overall_epoch_end(self, training_context):
+        if self.unit == 'epoch' and (training_context['epochs'] + 1) % self.frequency == 0 :
+            if is_in_ipython() and self.counter == self.clean_ipython_output_frequency:
+                display.clear_output(wait=True)
+                self.counter = 0
+            self.loss_history_list = []
+            self.metric_history_list = []
+            plotable_metric_names = OrderedDict()
+            for i in range(len(self.training_items.value_list)):
+                trainitem = self.training_items.value_list[i]
+                self.loss_history_list.append(trainitem.epoch_loss_history)
+                plotable_metric_names[i] = [k for k, v in trainitem._metrics.item_list if v.print_only == False]
+                self.metric_history_list.append(trainitem.epoch_metric_history)
+            self.counter += 1
+            fig = loss_metric_curve(self.loss_history_list, self.metric_history_list, metrics_names=plotable_metric_names,
+                                    legend=training_context['training_names'].value_list, calculate_base='epoch',
+                                    max_iteration=None, save_path=os.path.join(self.save_path, self.name_prefix),
+                                    imshow=self.imshow)
+            if ctx.enable_tensorboard and ctx.summary_writer is not None:
+                ctx.summary_writer.add_figure('overall/plot/loss_metric_curve', fig, global_step=training_context['steps'],
+                                              close=True, walltime=time.time())
+            plt.close()
 
     # def on_batch_end(self, training_context):
     #     if self.is_inplace:
@@ -536,12 +550,13 @@ class PrintGradientsCallback(VisualizationCallbackBase):
         self.first_layer = OrderedDict()
         self.last_layer = OrderedDict()
         self.is_modulefict = False
+        self.is_shared = True
         self.lines = []
 
     def on_optimization_step_start(self, training_context):
         if get_backend() == 'pytorch':
             with torch.no_grad():
-                if self.frequency > 0 and ((self.unit == 'batch' and (training_context['current_batch'] + 1) % self.frequency == 0) or (
+                if self.frequency > 0 and ((self.unit == 'batch' and (training_context['steps'] + 1) % self.frequency == 0) or (
                         self.unit == 'step' and (training_context['steps'] + 1) % self.frequency == 0) or (
                                                    self.unit == 'epoch' and (training_context['current_epoch'] + 1) % self.frequency == 0)):
                     grad_dict = OrderedDict()
@@ -605,7 +620,7 @@ class PrintGradientsCallback(VisualizationCallbackBase):
 
 
         elif get_backend() == 'tensorflow':
-            if self.frequency > 0 and ((self.unit == 'batch' and (training_context['current_batch'] + 1) % self.frequency == 0) or (
+            if self.frequency > 0 and ((self.unit == 'batch' and (training_context['steps'] + 1) % self.frequency == 0) or (
                     self.unit == 'step' and (training_context['steps'] + 1) % self.frequency == 0) or (
                                                self.unit == 'epoch' and training_context['current_batch'] == 0 and (training_context['current_epoch'] + 1) % self.frequency == 0)):
                 grad_dict = OrderedDict()
@@ -675,4 +690,36 @@ class PrintGradientsCallback(VisualizationCallbackBase):
             for line in self.lines:
                 print(line)
             self.lines = []
+    def on_overall_epoch_end(self, training_context):
+        if len(self.lines) > 0:
+            for line in self.lines:
+                ctx.print(line)
+            self.lines = []
 
+class PrintGpuUtilizationCallback(VisualizationCallbackBase):
+    def __init__(self, frequency=-1, unit='batch'):
+        super(PrintGpuUtilizationCallback, self).__init__(frequency, unit)
+        self.lines = []
+    def print_gpu_utilization(self):
+        memory_map_list=get_gpu_memory_map()
+        for map in memory_map_list:
+            self.lines.append(blue_color('[{0}] {1} '.format(map.value_list[1],map.value_list[2]))+'|'+orange_color(' {0:>3}°C {1:>3}% '.format(int(map.value_list[-2]),int(map.value_list[3])))+'|'+blue_color(' {0} /{1} MB  {2:.2%}'.format(int(map.value_list[4]),map.value_list[6],map.value_list[-1])))
+
+
+    def on_overall_batch_end(self, training_context):
+        if self.frequency > 0 and ((self.unit == 'batch' and (training_context['steps'] + 1) % self.frequency == 0) or (
+                self.unit == 'step' and (training_context['steps'] + 1) % self.frequency == 0)):
+            self.print_gpu_utilization()
+            if len(self.lines) > 0:
+                for line in self.lines:
+                    ctx.print(line)
+                self.lines = []
+
+    def on_overall_epoch_end(self, training_context):
+        if self.frequency > 0 and (self.unit == 'epoch' and training_context['current_batch'] == 0 and (
+                training_context['current_epoch'] + 1) % self.frequency == 0):
+            self.print_gpu_utilization()
+            if len(self.lines) > 0:
+                for line in self.lines:
+                    ctx.print(line)
+                self.lines = []
