@@ -16,6 +16,8 @@ from xml.etree import ElementTree
 import numpy as np
 import copy
 import cv2
+import datetime
+from trident.backend.common import *
 from trident.data.text_transforms import ToHalfWidth, ChineseConvert,bpmf_phonetic
 
 from trident.data.utils import _delete_h
@@ -30,7 +32,7 @@ from trident.data.utils import *
 from trident.backend.common import floatx,OrderedDict,is_alphabet,is_punctuation, remove_nonprintable
 from trident.backend.tensorspec import *
 from trident.misc.ipython_utils import *
-
+from trident import context
 try:
     from urllib.request import urlretrieve
 except ImportError:
@@ -39,7 +41,7 @@ except ImportError:
 __all__ = ['load_mnist', 'load_lfw', 'load_text', 'load_cifar', 'load_kaggle', 'load_birdsnap', 'load_examples_data', 'load_stanford_cars', 'load_folder_images']
 
 
-_session = get_session()
+_session = context._context()
 _trident_dir = os.path.join(_session.trident_dir, 'datasets')
 _backend = get_backend()
 
@@ -655,7 +657,6 @@ def load_examples_data(dataset_name):
                                           object_type=ObjectType.gray)
 
         dataset.testdata = dataset_test.traindata
-        dataset.class_names['zh-cn'] = dataset.class_names['en-us']
         return dataset
 
     elif dataset_name == 'examples_animals':
@@ -769,7 +770,7 @@ def load_examples_data(dataset_name):
                 return tuple(int(i) for i in b.split(' ')), a
 
         label_codes, label_names = zip(
-            *[parse_code(l) for l in open(os.path.join(dirname, "label_colors.txt")).readlines()])
+            *[parse_code(l) for l in open(os.path.join(dirname, "label_colors.txt"),encoding='utf-8-sig').readlines()])
         for i in range(len(label_codes)):
             mskdata.palette[label_names[i]] = label_codes[i]
 
