@@ -470,12 +470,18 @@ class TextSequenceDataProvider(object):
 
     def __init__(self, dataset_name='', traindata=None, testdata=None, batch_size=8,sequence_length=64, mode='tuple', **kwargs):
         self.__name__ = dataset_name
-        self.sequence_length = sequence_length
+        self._sequence_length = sequence_length
         self.uuid = uuid.uuid4().node
         if mode in ['tuple', 'dict']:
             self.mode = mode
         else:
             raise ValueError("Valid mode should be tuple or dict ")
+        self._text_transform_funcs = []
+        self._label_transform_funcs = []
+        self._paired_transform_funcs = []
+        self._batch_transform_funcs = []
+        cxt = context._context()
+        cxt.regist_data_provider(self)
 
         self.traindata = traindata
         if  isinstance(traindata,Iterator):
@@ -512,12 +518,7 @@ class TextSequenceDataProvider(object):
         self.tot_minibatch = 0
         self.tot_records = 0
         self.tot_epochs = 0
-        self._text_transform_funcs = []
-        self._label_transform_funcs = []
-        self._paired_transform_funcs = []
-        self._batch_transform_funcs = []
-        cxt = context._context()
-        cxt.regist_data_provider(self)
+
 
     @property
     def signature(self):
@@ -548,13 +549,20 @@ class TextSequenceDataProvider(object):
             return []
 
     @property
-    def batch_sampler(self):
-        if self.scenario == 'test' and self.testdata is not None:
-            return self.testdata.batch_sampler
-        elif self.traindata is not None:
-            return self.traindata.batch_sampler
-        else:
-            return []
+    def sequence_length(self):
+        return self._sequence_length
+
+    @sequence_length.setter
+    def sequence_length(self, value):
+        self._sequence_length=value
+        datasets=self.traindata.get_datasets()
+        for ds in datasets:
+            if isinstance(ds,TextSequenceDataset):
+                ds.sequence_length=value
+        if hasattr(self,'_text_transform_funcs') and len(self._text_transform_funcs)>0:
+            for tm in self._text_transform_funcs:
+                if isinstance(tm,VocabsMapping):
+                    tm.sequence_length=value
 
     @property
     def batch_size(self):
@@ -581,6 +589,7 @@ class TextSequenceDataProvider(object):
     def text_transform_funcs(self):
         return self._text_transform_funcs
 
+
     @text_transform_funcs.setter
     def text_transform_funcs(self, value):
         self._text_transform_funcs = value
