@@ -20,11 +20,12 @@ import numpy as np
 from trident import context
 from trident.backend import iteration_tools
 from trident.backend.common import *
-from trident.backend.numpy_ops import DTYPE_MAPPING
+#from trident.backend.numpy_ops import DTYPE_MAPPING
+from trident.backend import dtype as Dtype
 from trident.backend.opencv_backend import file2array
-from trident.backend.tensorspec import TensorSpec
+from trident.backend.tensorspec import TensorSpec,ObjectType
 from trident.data.image_common import image_backend_adaption, reverse_image_backend_adaption, \
-    array2image
+    array2image,TensorShape
 from trident.data.label_common import label_backend_adaptive
 from trident.data.mask_common import mask_backend_adaptive, color2label
 from trident.data.samplers import *
@@ -38,11 +39,9 @@ except ImportError:
     import queue as Queue
 
 if get_backend() == 'pytorch':
-    from trident.backend.pytorch_backend import to_tensor, ObjectType
-    from trident.backend.pytorch_ops import tensor_to_shape, expand_dims, cast
+    from trident.backend.pytorch_ops import tensor_to_shape, expand_dims, cast,to_tensor
 elif get_backend() == 'tensorflow':
-    from trident.backend.tensorflow_backend import to_tensor, ObjectType
-    from trident.backend.tensorflow_ops import tensor_to_shape, expand_dims, cast
+    from trident.backend.tensorflow_ops import tensor_to_shape, expand_dims, cast,to_tensor
 
 __all__ = ['Dataset', 'ZipDataset', 'ImageDataset', 'MaskDataset', 'TextSequenceDataset', 'LabelDataset', 'BboxDataset', 'LandmarkDataset', 'Iterator', 'MetricIterator',
            'NumpyDataset', 'RandomNoiseDataset', 'ArrayDataset']
@@ -315,46 +314,50 @@ class ImageDataset(Dataset):
         self.is_paired_process = False
 
     def __getitem__(self, index: int):
-        if index >= len(self.items):
-            index = index % len(self.items)
-        img = self.items[index]  # self.pop(index)
+        try:
+            if index >= len(self.items):
+                index = index % len(self.items)
+            img = self.items[index]  # self.pop(index)
 
-        if isinstance(img, str) and self.object_type == ObjectType.image_path:
-            return img
-        elif isinstance(img, np.ndarray) and self.object_type != ObjectType.image_path:
-            return img
-        else:
-            img = file2array(img)
-
-        if not isinstance(img, np.ndarray):
-            raise ValueError('image data should be ndarray')
-        elif isinstance(img, np.ndarray) and img.ndim not in [2, 3]:
-            raise ValueError('image data dimension  should be 2 or 3, but get {0}'.format(img.ndim))
-        elif self.object_type == ObjectType.gray:
-            if img.ndim == 2:
-                img = np.expand_dims(img, -1)
-            elif (img.ndim == 3 and img.shape[-1] == 1):
-                pass
+            if isinstance(img, str) and self.object_type == ObjectType.image_path:
+                return img
+            elif isinstance(img, np.ndarray) and self.object_type != ObjectType.image_path:
+                return img
             else:
-                img = cv2.cvtColor(img, cv2.COLOR_RGB2GRAY)
-                img = np.expand_dims(img, -1)
-        elif self.object_type == ObjectType.rgb and img.ndim == 2:
-            img = cv2.cvtColor(img, cv2.COLOR_GRAY2RGB)
-        elif self.object_type == ObjectType.rgb and img.ndim == 3:
-            if img.shape[-1] == 1:
-                img = cv2.cvtColor(img[:, :, 0], cv2.COLOR_GRAY2RGB)
-            elif img.shape[-1] == 4:
-                img = cv2.cvtColor(img, cv2.COLOR_RGBA2RGB)
-            img = img[:, :, :3]
+                img = file2array(img)
 
-        elif self.object_type == ObjectType.rgba:
-            if img.ndim == 2:
-                img = cv2.cvtColor(img, cv2.COLOR_GRAY2RGBA)
-            if img.shape[2] == 3:
-                img = cv2.cvtColor(img, cv2.COLOR_RGB2RGBA)
-        elif self.object_type == ObjectType.multi_channel:
-            img = img.astype(np.float32)
-        return img
+            if not isinstance(img, np.ndarray):
+                raise ValueError('image data should be ndarray')
+            elif isinstance(img, np.ndarray) and img.ndim not in [2, 3]:
+                raise ValueError('image data dimension  should be 2 or 3, but get {0}'.format(img.ndim))
+            elif self.object_type == ObjectType.gray:
+                if img.ndim == 2:
+                    img = np.expand_dims(img, -1)
+                elif (img.ndim == 3 and img.shape[-1] == 1):
+                    pass
+                else:
+                    img = cv2.cvtColor(img, cv2.COLOR_RGB2GRAY)
+                    img = np.expand_dims(img, -1)
+            elif self.object_type == ObjectType.rgb and img.ndim == 2:
+                img = cv2.cvtColor(img, cv2.COLOR_GRAY2RGB)
+            elif self.object_type == ObjectType.rgb and img.ndim == 3:
+                if img.shape[-1] == 1:
+                    img = cv2.cvtColor(img[:, :, 0], cv2.COLOR_GRAY2RGB)
+                elif img.shape[-1] == 4:
+                    img = cv2.cvtColor(img, cv2.COLOR_RGBA2RGB)
+                img = img[:, :, :3]
+
+            elif self.object_type == ObjectType.rgba:
+                if img.ndim == 2:
+                    img = cv2.cvtColor(img, cv2.COLOR_GRAY2RGBA)
+                if img.shape[2] == 3:
+                    img = cv2.cvtColor(img, cv2.COLOR_RGB2RGBA)
+            elif self.object_type == ObjectType.multi_channel:
+                img = img.astype(np.float32)
+            return img
+        except Exception as e:
+            print(e)
+            PrintException()
 
     def data_transform(self, img_data):
         if self.object_type == ObjectType.image_path:
@@ -394,7 +397,9 @@ class ImageDataset(Dataset):
     def reverse_image_transform(self, img_data):
         if len(self.reverse_image_transform_funcs) == 0:
             return reverse_image_backend_adaption(img_data)
-        if isinstance(img_data, np.ndarray):
+        if self.object_type==ObjectType.image_path:
+            return img_data
+        elif isinstance(img_data, np.ndarray):
             # if img_data.ndim>=2:
             for fc in self.reverse_image_transform_funcs:
                 if (inspect.isfunction(fc) or isinstance(fc, Transform)) and fc is not reverse_image_backend_adaption:
@@ -1001,6 +1006,7 @@ class TextSequenceDataset(Dataset):
             for i in range(self.sequence_length):
                 if i < len(text_data):
                     this_char = text_data[i]
+
                     if this_char in self.text2index:
                         arr[i, self.text2index[this_char]] = 1
                     else:
@@ -1016,7 +1022,7 @@ class TextSequenceDataset(Dataset):
                     if this_char in self.text2index:
                         arr[i] = self.text2index[this_char]
                     else:
-                        arr[i] = self.text2index['[UNK]']
+                        arr[i] = self.text2index['']
                 else:
                     arr[i] = self.text2index['[PAD]']
             arr = arr.astype(np.int64)
@@ -1106,7 +1112,7 @@ class Iterator(object):
         for k in range(len(datasets)):
             ds = datasets[k]
             if isinstance(ds, TextSequenceDataset):
-                ds.element_spec = TensorSpec(shape=TensorShape([None, ds.sequence_length]), dtype=dtype.long, object_type=ds.object_type, name=ds.symbol)
+                ds.element_spec = TensorSpec(shape=TensorShape([None, ds.sequence_length]), dtype=Dtype.long, object_type=ds.object_type, name=ds.symbol)
             elif ds.object_type == ObjectType.image_path:
                 ds.element_spec = TensorSpec(shape=TensorShape([None]), dtype=str, object_type=ds.object_type, name=ds.symbol)
             else:
