@@ -25,6 +25,10 @@ try:
 except ImportError:
     from six.moves.urllib.request import urlretrieve
 
+__all__: object = ['is_connected', 'ensure_dir','ensure_parent_dir','TqdmProgress','calculate_md5','check_integrity','download_file','get_onedrive_directdownload',
+           'download_file_from_google_drive','download_file_from_onedrive','get_image_from_google_drive','get_file_from_google_drive',
+           'download_model_from_google_drive','download_model_from_onedrive','extract_archive','pickle_it','unpickle','save_dict_as_h5',
+          'read_dict_from_h5','get_file_create_time','read_mat']
 
 def is_connected():
     try:
@@ -573,10 +577,11 @@ def extract_archive(file_path, target_folder=None, archive_format='auto'):
         True if a match was found and an archive extraction was completed,
         False otherwise.
     """
+    folder, file, ext = split_path(file_path)
+    filename = file + ext
     try:
-        folder, file, ext = split_path(file_path)
+
         _h = _read_h(target_folder)
-        filename = file + ext
         is_extracted = _h[filename].get('is_extracted', False) if filename in _h else _h.get('is_extracted', False)
         if _h != {} and is_extracted == True and os.path.exists(file_path):
             print('extraction is finished, donnot need extract again.')
