@@ -197,8 +197,8 @@ class Resize(VisionTransform):
         coords[:, 0] = np.round(coords[:, 0] * (float(tw) / w))
         coords[:, 1] = np.round(coords[:, 1] * (float(th) / h))
         if not self.align_corner:
-            coords[:, 0] += pad_horz // 2
-            coords[:, 1] += pad_vert // 2
+            coords[:, 0] += pad_vert // 2
+            coords[:, 1] += pad_horz // 2
         return coords
 
     def _apply_mask(self, mask, spec: TensorSpec):
@@ -372,9 +372,11 @@ class RandomRescaleCrop(VisionTransform):
         self.output_size = output_size
         if isinstance(self.output_size, numbers.Number):
             self.output_size = (output_size, output_size)
-        self.scale_range = scale_range
-        if isinstance(self.scale_range, numbers.Number):
+        scale_range =kwargs.get('scale',scale_range)
+        if isinstance(scale_range, numbers.Number):
             self.scale_range = (scale_range, scale_range)
+        else:
+            self.scale_range =scale_range
 
         self.interpolation = interpolation
 
@@ -461,9 +463,11 @@ class RandomCenterCrop(VisionTransform):
         self.output_size = output_size
         if isinstance(self.output_size, numbers.Number):
             self.output_size = (output_size, output_size)
-        self.scale_range = scale_range
-        if isinstance(self.scale_range, numbers.Number):
+        scale_range =kwargs.get('scale',scale_range)
+        if isinstance(scale_range, numbers.Number):
             self.scale_range = (scale_range, scale_range)
+        else:
+            self.scale_range =scale_range
         self.interpolation = interpolation
 
     def apply(self, input: Tuple, spec: TensorSpec):
@@ -715,8 +719,9 @@ class RandomTransformAffine(VisionTransform):
             mask = mask.astype(np.float32)
 
             mask = cv2.warpAffine(mask, mat_img, (width, height), borderMode=cv2.BORDER_CONSTANT, borderValue=(0, 0, 0),flags=cv2.INTER_NEAREST)  # , borderMode=cv2.BORDER_REPLICATE
-            mask[mask>=0.5]=1
-            mask[mask < 0.5] =0
+            if spec.object_type==ObjectType.binary_mask:
+                mask[mask>=0.5]=1
+                mask[mask < 0.5] =0
             mask = mask.astype(mask_dtype)
             if is_flip:
                 return np.fliplr(mask)
@@ -769,7 +774,11 @@ class RandomMultiScaleImage(VisionTransform):
         self.output_size = output_size
         if isinstance(self.output_size, numbers.Number):
             self.output_size = (output_size, output_size)
-        self.scale_range = scale_range
+        scale_range = kwargs.get('scale', scale_range)
+        if isinstance(scale_range, numbers.Number):
+            self.scale_range = (scale_range, scale_range)
+        else:
+            self.scale_range = scale_range
         self.interpolation = interpolation
         self.idx = 0
         self.tmp_fun=None
@@ -1299,6 +1308,10 @@ class GrayScale(VisionTransform):
             return cv2.cvtColor(cv2.cvtColor(image.astype(np.float32), cv2.COLOR_RGBA2GRAY), cv2.COLOR_GRAY2RGB)
         elif image.ndim == 4 and not self.keepdims:
             return cv2.cvtColor(image.astype(np.float32), cv2.COLOR_RGBA2GRAY)
+        elif image.ndim == 3 and image.shape[-1]==1 and self.keepdims:
+            return image
+        elif image.ndim == 3 and image.shape[-1]==1 and not self.keepdims:
+            return image[:,:,0]
         elif image.ndim == 3 and self.keepdims:
             return cv2.cvtColor(cv2.cvtColor(image.astype(np.float32), cv2.COLOR_RGB2GRAY), cv2.COLOR_GRAY2RGB)
         elif image.ndim == 3 and not self.keepdims:
@@ -1534,7 +1547,7 @@ class AdaptiveBinarization(VisionTransform):
         return super().apply(input, spec)
 
     def _apply_image(self, image, spec: TensorSpec):
-        if image.ndim == 3:
+        if image.ndim == 3 and image.shape[-1]!=1:
             gray = cv2.cvtColor(image.astype(np.uint8), cv2.COLOR_RGB2GRAY).astype(np.uint8)
         else:
             gray = image.astype(np.uint8)
