@@ -61,6 +61,7 @@ class RNNBase(Layer):
         self.num_layers = num_layers
         self.use_bias = use_bias
         self.stateful=stateful
+        self.hx=None
         self._batch_first = batch_first
         if not self._batch_first:
             self.batch_index =1
@@ -109,8 +110,25 @@ class RNNBase(Layer):
             else:
                 self.batch_index = 0
 
+    def clear_state(self):
+        self.hx = None
+
     def initial_state(self,input) :
-        pass
+        max_batch_size = input.size(0) if self.batch_first else input.size(1)
+        num_directions = 2 if self.bidirectional else 1
+
+        h_zeros=torch.zeros(self.num_layers * num_directions,
+                                  max_batch_size, self.hidden_size,
+                                  dtype=input.dtype, device=input.device).to(get_device())
+        c_zeros= torch.zeros(self.num_layers * num_directions,
+                                  max_batch_size, self.hidden_size,
+                                  dtype=input.dtype, device=input.device).to(get_device())
+        if self.stateful:
+            self.hx = (h_zeros, c_zeros)
+            return self.hx
+        else:
+            return (h_zeros, c_zeros)
+
 
     def build(self, input_shape:TensorShape):
         if not self._built:
@@ -586,28 +604,16 @@ class LSTM(RNNBase):
         >>> output, (hn, cn) = rnn(input, (h0, c0))
     """
 
-    def __init__(self, hidden_size,proj_size=0,num_layers:int =2,activation=None,stateful=False,use_bias=False,use_attention=False,attention_size=16,batch_first=False,dropout_rate=0,bidirectional=False,keep_output=False,name=None, **kwargs):
+    def __init__(self, hidden_size,proj_size=0,num_layers:int =2,stateful=False,use_bias=False,use_attention=False,attention_size=16,batch_first=False,dropout_rate=0,bidirectional=False,keep_output=False,name=None, **kwargs):
         super(LSTM, self).__init__(mode='LSTM', hidden_size=hidden_size, proj_size=proj_size,
         num_layers=num_layers, stateful = stateful, use_bias=use_bias, batch_first = batch_first,
         dropout_rate= dropout_rate, bidirectional = bidirectional, keep_output =keep_output, in_sequence = True, filter_index =-1, name = name)
-        self.hx=None
+
         self.use_attention=use_attention
         self.attention_size=attention_size
 
-    def clear_state(self):
-        self.hx = None
-    def initial_state(self,input) :
-        max_batch_size = input.size(0) if self.batch_first else input.size(1)
-        num_directions = 2 if self.bidirectional else 1
 
-        h_zeros=torch.zeros(self.num_layers * num_directions,
-                                  max_batch_size, self.hidden_size,
-                                  dtype=input.dtype, device=input.device).to(get_device())
-        c_zeros= torch.zeros(self.num_layers * num_directions,
-                                  max_batch_size, self.hidden_size,
-                                  dtype=input.dtype, device=input.device).to(get_device())
-        self.hx = (h_zeros, c_zeros)
-        return self.hx
+
 
 
 
@@ -624,11 +630,13 @@ class LSTM(RNNBase):
     def check_forward_args(self, input: Tensor, hidden: Tuple[Tensor, Tensor], batch_sizes: Optional[Tensor]):
         self.check_input(input, batch_sizes)
         expected_hidden_size = self.get_expected_hidden_size(input, batch_sizes)
-
-        self.check_hidden_size(hidden[0], self.get_expected_hidden_size(input, batch_sizes),
-                               'Expected hidden[0] size {}, got {}')
-        self.check_hidden_size(hidden[1], self.get_expected_cell_size(input, batch_sizes),
-                               'Expected hidden[1] size {}, got {}')
+        try:
+            self.check_hidden_size(hidden[0], self.get_expected_hidden_size(input, batch_sizes),
+                                   'Expected hidden[0] size {}, got {}')
+            self.check_hidden_size(hidden[1], self.get_expected_cell_size(input, batch_sizes),
+                                   'Expected hidden[1] size {}, got {}')
+        except:
+            self.initial_state(input)
 
     def permute_hidden(self, hx: Tuple[Tensor, Tensor], permutation: Optional[Tensor]) -> Tuple[Tensor, Tensor]:
         if permutation is None:
@@ -662,6 +670,10 @@ class LSTM(RNNBase):
         pass
 
     def forward(self, x, hx=None):
+        # helper to inject peephole connection if requested
+        # def peep(x, c, C):
+        #     return x + C * c if use_peepholes else x
+        #
         orig_input = x
         is_packed_sequence=isinstance(orig_input, PackedSequence)
         self.flatten_parameters()
@@ -673,24 +685,45 @@ class LSTM(RNNBase):
         else:
             if not self.batch_first:
                 x = x.transpose(1,0)
-            batch_sizes = None
+
+            batch_sizes = None #x.size(0) if self.batch_first else x.size(1)
             max_batch_size = x.size(0) if self.batch_first else x.size(1)
             sorted_indices = None
             unsorted_indices = None
 
-        if hx is None:
+        if not self.stateful:
             #if self.hidden_state is None or self.cell_state is None or max_batch_size!=int_shape(self.hidden_state)[1]:
             hx=self.initial_state(x)
-            hx = self.permute_hidden(hx, sorted_indices)
-        else:
-            if not self.stateful:
-                hx=self.initial_state(x)
-            hx = self.permute_hidden(hx, sorted_indices)
+        elif self.stateful and self.hx is None and hx is None:
+            hx = self.initial_state(x)
+        elif self.stateful and self.hx is None and hx is not None:
+            self.hx=hx
+        elif self.stateful and self.hx is not None:
+            if batch_sizes is not None:
+                mini_batch = int(batch_sizes[0])
+            else:
+                mini_batch = x.size(0) if self.batch_first else x.size(1)
+            #mini_batch size changed need re-initial hidden cell
+            if  self.hx[0].size(1)!=mini_batch:
+                hx = self.initial_state(x)
+            else:
+                hx=self.hx
+
+
+
+        hx = self.permute_hidden(hx, sorted_indices)
 
         self.check_forward_args(x, hx, batch_sizes)
 
-        if not isinstance(x, PackedSequence):
-            result = _VF.lstm(x,hx, self._flat_weights, self.use_bias, self.num_layers,
+        # if not isinstance(x, PackedSequence):
+        #     result = _VF.lstm(x,hx, self._flat_weights, self.use_bias, self.num_layers,
+        #                       self.dropout_rate, self.training, self.bidirectional, self.batch_first)
+        # else:
+        #     result = _VF.lstm(x, batch_sizes, hx, self._flat_weights, self.use_bias,
+        #                       self.num_layers, self.dropout_rate, self.training, self.bidirectional)
+        #
+        if batch_sizes is None:
+            result = _VF.lstm(x, hx, self._flat_weights, self.use_bias, self.num_layers,
                               self.dropout_rate, self.training, self.bidirectional, self.batch_first)
         else:
             result = _VF.lstm(x, batch_sizes, hx, self._flat_weights, self.use_bias,
@@ -698,12 +731,14 @@ class LSTM(RNNBase):
 
 
         output = result[0].permute(1, 0, 2) if self.batch_first == False else result[0]
-        hidden = result[1:]
-        # self.hidden_state=hidden[0]
-        # self.cell_state=hidden[1]
         if self.use_attention:
             output = self.attention(output)
 
+        hidden = result[1:]
+        hidden=tuple([item.detach() for item in hidden])
+        if self.stateful:
+            self.hx = hidden
+
         # xxx: isinstance check needs to be in conditional for TorchScript to compile
         if is_packed_sequence:
             output_packed = PackedSequence(output, batch_sizes, sorted_indices, unsorted_indices)
