@@ -350,12 +350,12 @@ class BERT(Layer):
         for i in range(n_layers):
             self.add_module('transformer_block{0}'.format(i),TransformerBlock(hidden, attn_heads, hidden * 4, dropout_rate) )
 
-    def forward(self, x):
+    def forward(self, x,segments_tensor=None):
         if int_shape(x)[1]==2:
             x,segments_tensor=split(x,num_splits=2,axis=1)
             x=x.squeeze(1)
             segments_tensor=segments_tensor.squeeze(1)
-        else:
+        elif segments_tensor is None:
             segments_tensor = zeros_like(x, dtype=x.dtype).to(get_device())
         # attention masking for padded token
         # torch.ByteTensor([batch_size, 1, seq_len, seq_len)
