@@ -50,7 +50,7 @@ class HistoryBase(OrderedDict):
         if data_name not in self:
             self[data_name]=[]
 
-    def collect(self, data_name: str, step: int, value: (float,np.ndarray, Tensor)):
+    def collect(self, data_name: str, step: int, value: (float,np.ndarray)):
         if data_name not in self:
             self.regist(data_name)
         if any_abnormal_number(value):
