@@ -6,7 +6,7 @@ import builtins
 import sys
 
 
-from trident.backend.common import if_none
+from trident.backend.common import if_none,get_plateform
 from trident.backend.opencv_backend import array2image
 from trident.misc.ipython_utils import is_in_ipython, is_in_colab
 import math
@@ -25,8 +25,23 @@ import matplotlib.pyplot as plt
 from matplotlib.collections import PolyCollection
 import matplotlib.patches as patches
 import matplotlib.font_manager
+import PIL
+from PIL import Image
+from PIL import ImageDraw
+from PIL import ImageFont
 
-# fonts = matplotlib.font_manager.findSystemFonts(fontpaths=None, fontext='ttf')
+
+
+fonts = matplotlib.font_manager.findSystemFonts(fontpaths=None, fontext='ttf')
+default_font = None
+if get_plateform()== 'linux':
+    candidate=[f for f in fonts if 'Ubuntu-LI.ttf' in f or 'LiberationSans-Regular.ttf' in f]
+    default_font=candidate[0] if len(candidate)>0 else  fonts[0]
+elif get_plateform()== 'windows':
+    candidate = [f for f in fonts if 'Microsoft Sans Serif' in matplotlib.font_manager.FontProperties(fname=f).get_name()  or 'heiti' in matplotlib.font_manager.FontProperties(fname=f).get_name() ]
+    default_font = candidate[0] if len(candidate) > 0 else fonts[0]
+# ImageFont.truetype(f, 20)
+#if
 # fontnames = [matplotlib.font_manager.FontProperties(fname=fname).get_name() for fname in fonts]
 # default_font = None
 #
@@ -39,10 +54,6 @@ import matplotlib.font_manager
 #                 default_font = matplotlib.rc('font', family=name)
 #                 break
 
-import PIL
-from PIL import Image
-from PIL import ImageDraw
-from PIL import ImageFont
 
 import colorsys
 import itertools
@@ -111,17 +122,21 @@ def plot_bbox(box, img, color=None, label=None, line_thickness=None, **kwargs):
         draw.rectangle(((box[0], box[1]), (box[2], box[3])), outline=color, fill=None, width=tl)
         fontcolor = (255, 255, 255)
         avg_color = np.array(list(color)).mean()
-        font=None#ImageFont.truetype(fonts[fontnames.index('Hiragino Sans GB')],int(math.sqrt(img_shape[0] / 1000) * 10 + 1))
-        if avg_color > 150:
+        min_color= np.array(list(color)).min()
+        font=ImageFont.truetype(default_font, int(math.sqrt(img_shape[0] / 1000) * 16+1 ))
+        #font=None#ImageFont.truetype(fonts[fontnames.index('Hiragino Sans GB')],int(math.sqrt(img_shape[0] / 1000) * 10 + 1))
+        if avg_color >=120 or min_color<=32:
             fontcolor = (0, 0, 0)
-        if label and get_plateform() == 'windows':
-            font = ImageFont.truetype(fonts[fontnames.index('Microsoft Sans Serif')], int(math.sqrt(img_shape[0] / 1000) * 10 + 1))
+        if label :
+            #font = ImageFont.truetype(fonts[fontnames.index('Microsoft Sans Serif')], int(math.sqrt(img_shape[0] / 1000) * 10 + 1))
             tf = max(tl - 1, 1)  # font thickness
-            t_size = draw.textsize(label, font=font)
-            c2 = c1[0] + t_size[0], c1[1] - t_size[1] - 3
-            offset = font.getoffset(label)
-            draw.rectangle((c1, c2), fill=color,width=2)
-            draw.text((c1[0], c1[1] - 2- offset[1] ), u'{0}'.format(label), fill=fontcolor, font=font)
+            t_size = draw.textsize(str(label), font=font)
+            offset = font.getoffset(str(label))
+            c2 = c1[0] , c1[1] - t_size[1] - 2*offset[1] - 3
+            c3= c1[0] + 2*offset[0] + t_size[0]+3, c1[1]
+
+            draw.rectangle((c2, c3), fill=color,width=2)
+            draw.text((c1[0]+1, c1[1] - t_size[1] - offset[1] - 2), ' {0}'.format(label), fill=fontcolor, font=font)
     except Exception as e:
         print('image_size', img_shape,box)
         print(e)
@@ -201,21 +216,20 @@ def loss_metric_curve(losses, metrics,metrics_names,legend=None, calculate_base=
     plt.ion()  # is not None:
 
     loss_ax1 = fig.add_subplot(2, 2, 1)
-
+    losses=unpack_singleton(losses)
+    only_single_model=True
     if losses.__class__.__name__ == 'HistoryBase':
         steps, values = losses.get_series('total_losses')
         loss_ax1.plot(steps, values, label='total_losses')
 
     elif isinstance(losses, list):
+        only_single_model=False
         for n in range(len(losses)):
             item = losses[n]
-            legend_label = 'total_losses' + str(n)
-            if legend is not None and len(legend) == len(losses):
-                legend_label = legend[n]
-
             if item.__class__.__name__ == 'HistoryBase':
                 steps, values = item.get_series('total_losses')
-                p = loss_ax1.plot(steps, values, label=legend_label)
+
+                p = loss_ax1.plot(steps, values, label= 'total_losses' + str(n))
                 colors.append(p[-1].get_color())
 
     loss_ax1.set_title('model loss', fontsize=14, fontweight='bold')
@@ -239,19 +253,17 @@ def loss_metric_curve(losses, metrics,metrics_names,legend=None, calculate_base=
         second_axis_keys = []
         first_axis_limit = []
         second_axis_limit = []
+        metrics=unpack_singleton(metrics)
         if metrics.__class__.__name__ == 'HistoryBase':
-            metrics_need_plot = metrics_names
+            metrics_need_plot = metrics_names[0] if isinstance(metrics_names,dict) else metrics_names
             if 'epoch' in metrics_need_plot:
                 metrics_need_plot.remove('epoch')
             for n in range(len(metrics)):
                 k, v = list(metrics.items())[n]
-
-                legend_label = legend[n] if legend is not None else None
-
                 if k in metrics_need_plot:
-                    legend_label = if_none(legend_label, k)
-
+                    legend_label = k
                     steps, values = metrics.get_series(k)
+
                     values_np = np.array(values)
                     if first_axis_range is None:
                         first_axis_range = (values_np.min(), values_np.mean(), values_np.max())
@@ -305,7 +317,9 @@ def loss_metric_curve(losses, metrics,metrics_names,legend=None, calculate_base=
                             if legend is not None and len(legend) == len(metrics):
                                 legend_label = legend[i]+' '+k
 
+
                             steps, values = item.get_series(k)
+
                             values_np = np.array(values)
                             if first_axis_range is None:
                                 first_axis_range = (values_np.min(), values_np.mean(), values_np.max())
@@ -512,12 +526,14 @@ def steps_histogram(grads, weights=None, sample_collected=None, bins=None, size=
         max_frequency = 0
 
         for i in range(len(grads)):
-            a, b = np.histogram(grads[i].reshape([-1]), bins)
-            ys = a
-            xs = b[:-1]
-            new_zs.append(zs[i])
-            max_frequency = max(np.max(a), max_frequency)
-            verts.append(polygon_under_graph(xs, ys))
+            if (i+1)%inteval==0:
+                a, b = np.histogram(grads[i].reshape([-1]), bins)
+                ys = a
+                xs = b[:-1]
+                new_zs.append(zs[i])
+                max_frequency = max(np.max(a), max_frequency)
+                verts.append(polygon_under_graph(xs, ys))
+
 
         poly = PolyCollection(verts, facecolors=['r', 'g', 'b', 'y'], alpha=.4)
         ax.add_collection3d(poly, zs=new_zs, zdir='y')
@@ -545,7 +561,7 @@ def steps_histogram(grads, weights=None, sample_collected=None, bins=None, size=
         new_zs = []
         max_frequency = 0
         for i in range(len(weights)):
-            if i % inteval == 0:
+            if (i + 1) % inteval == 0:
                 a, b = np.histogram(weights[i].reshape([-1]), bins)
                 ys = a
                 xs = b[:-1] + 0.001
