@@ -173,7 +173,7 @@ def basic_rfb(num_filters, scale=0.1):
                              activation=None,
                              normalization='batch'),
                 Conv2d_Block((3, 3), depth_multiplier=2, strides=1, groups=1, padding=(1, 1), use_bias=False,
-                             activation='relu',
+                             activation=Relu(inplace=True),
                              normalization='batch'),
                 Conv2d_Block((3, 3), depth_multiplier=1, strides=1, groups=1, padding=(2, 2), use_bias=False,
                              dilation=2,
@@ -184,7 +184,7 @@ def basic_rfb(num_filters, scale=0.1):
                              activation=None,
                              normalization='batch'),
                 Conv2d_Block((3, 3), depth_multiplier=2, strides=1, groups=1, padding=(1, 1), use_bias=False,
-                             activation='relu',
+                             activation=Relu(inplace=True),
                              normalization='batch'),
                 Conv2d_Block((3, 3), depth_multiplier=1, strides=1, groups=1, padding=(3, 3), use_bias=False,
                              dilation=3,
@@ -194,10 +194,10 @@ def basic_rfb(num_filters, scale=0.1):
                              activation=None,
                              normalization='batch'),
                 Conv2d_Block((3, 3), depth_multiplier=1.5, strides=1, groups=1, padding=(1, 1), use_bias=False,
-                             activation='relu',
+                             activation=Relu(inplace=True),
                              normalization='batch'),
                 Conv2d_Block((3, 3), depth_multiplier=1.33, strides=1, groups=1, padding=(1, 1), use_bias=False,
-                             activation='relu', normalization='batch'),
+                             activation=Relu(inplace=True), normalization='batch'),
                 Conv2d_Block((3, 3), depth_multiplier=1, strides=1, groups=1, padding=(5, 5), use_bias=False,
                              dilation=5,
                              activation=None, normalization='batch'), name='branch3')
@@ -211,16 +211,16 @@ def basic_rfb(num_filters, scale=0.1):
 
 def conv_dw(num_filters, strides):
     return Sequential(
-        DepthwiseConv2d_Block((3, 3), depth_multiplier=1, strides=strides, use_bias=False, activation='relu',
+        DepthwiseConv2d_Block((3, 3), depth_multiplier=1, strides=strides, use_bias=False, activation=Relu(inplace=True),
                               normalization='batch'),
         Conv2d_Block((1, 1), num_filters=num_filters, strides=1, groups=1, auto_pad=True, use_bias=False,
-                     activation='relu', normalization='batch'),
+                     activation=Relu(inplace=True), normalization='batch'),
     )
 
 
 def tiny_mobile_rfbnet(filter_base=16, num_classes=2):
     return Sequential(Conv2d_Block((3, 3), num_filters=filter_base, strides=2, groups=1, auto_pad=True, use_bias=False,
-                                   activation='relu', normalization='batch'),
+                                   activation=Relu(inplace=True), normalization='batch'),
                       conv_dw(filter_base * 2, 1),
                       conv_dw(filter_base * 2, 2),  # 80*60
                       conv_dw(filter_base * 2, 1),
@@ -256,48 +256,48 @@ class RFBnet(Layer):
         self.detection_threshold = detection_threshold
         self.nms_threshold = nms_threshold
         self.variance = (center_variance, size_variance)
-        self.softmax = SoftMax(axis=-1)
+        self.softmax=SoftMax(-1)
+
 
         self.num_classes = num_classes
         self.num_regressors = num_regressors
         self.register_buffer("priors", None)
         self.define_img_size(640)
 
-        self.extra = Sequential(Conv2d((1, 1), num_filters=64, strides=1, activation='relu', use_bias=True),
-                                DepthwiseConv2d((3, 3), depth_multiplier=1, strides=2, auto_pad=True, activation='relu',
+        self.extra = Sequential(Conv2d((1, 1), num_filters=64, strides=1, activation=Relu(inplace=True), use_bias=True),
+                                DepthwiseConv2d((3, 3), depth_multiplier=1, strides=2, auto_pad=True, activation=Relu(inplace=True),
                                                 use_bias=True),
-                                Conv2d((1, 1), num_filters=256, strides=1, activation=None, use_bias=True), Relu(),
+                                Conv2d((1, 1), num_filters=256, strides=1, activation=None, use_bias=True), Relu(inplace=True),
                                 name='extra')
         self.regression_headers = ModuleList([
             Sequential(
-            DepthwiseConv2d((3, 3), depth_multiplier=1, strides=1, auto_pad=True, activation='relu', use_bias=True),
+            DepthwiseConv2d((3, 3), depth_multiplier=1, strides=1, auto_pad=True, activation=Relu(inplace=True), use_bias=True),
             Conv2d((1, 1), num_filters=3 * self.num_regressors, strides=1, activation=None, use_bias=True)),
 
             Sequential(
-                DepthwiseConv2d((3, 3), depth_multiplier=1, strides=1, auto_pad=True, activation='relu', use_bias=True),
+                DepthwiseConv2d((3, 3), depth_multiplier=1, strides=1, auto_pad=True, activation=Relu(inplace=True), use_bias=True),
                 Conv2d((1, 1), num_filters=2 * self.num_regressors, strides=1, activation=None, use_bias=True)),
 
             Sequential(
-                DepthwiseConv2d((3, 3), depth_multiplier=1, strides=1, auto_pad=True, activation='relu', use_bias=True),
+                DepthwiseConv2d((3, 3), depth_multiplier=1, strides=1, auto_pad=True, activation=Relu(inplace=True), use_bias=True),
                 Conv2d((1, 1), num_filters=2 * self.num_regressors, strides=1, activation=None, use_bias=True)),
 
             Conv2d((3, 3), num_filters=3 * self.num_regressors, strides=1, auto_pad=True, activation=None) ],
             name='regression_headers')
         self.classification_headers = ModuleList([
             Sequential(
-                DepthwiseConv2d((3, 3), depth_multiplier=1, strides=1, auto_pad=True, activation='relu', use_bias=True),
+                DepthwiseConv2d((3, 3), depth_multiplier=1, strides=1, auto_pad=True, activation=Relu(inplace=True), use_bias=True),
                 Conv2d((1, 1), num_filters=3 * self.num_classes, strides=1, activation=None, use_bias=True)),
 
             Sequential(
-                DepthwiseConv2d((3, 3), depth_multiplier=1, strides=1, auto_pad=True, activation='relu', use_bias=True),
+                DepthwiseConv2d((3, 3), depth_multiplier=1, strides=1, auto_pad=True, activation=Relu(inplace=True), use_bias=True),
                 Conv2d((1, 1), num_filters=2 * self.num_classes, strides=1, activation=None, use_bias=True)),
 
             Sequential(
-                DepthwiseConv2d((3, 3), depth_multiplier=1, strides=1, auto_pad=True, activation='relu', use_bias=True),
-                Conv2d((1, 1), num_filters=2 * self.num_classes, strides=1, activation=None, use_bias=True)),
+                DepthwiseConv2d((3, 3), depth_multiplier=1, strides=1, auto_pad=True, activation=Relu(inplace=True), use_bias=True),
+                Conv2d((1, 1), num_filters=2 * self.num_classes, strides=1, auto_pad=True, activation=None, use_bias=True)),
 
-            Conv2d((3, 3), num_filters=3 * self.num_classes, strides=1, auto_pad=True, activation=None,
-                   use_bias=True) ], name='classification_headers')
+            Conv2d((3, 3), num_filters=3 * self.num_classes, strides=1, auto_pad=True, activation=None, use_bias=True) ], name='classification_headers')
 
     def define_img_size(self, size=640):
         global image_size, feature_map_w_h_list, priors
@@ -323,16 +323,9 @@ class RFBnet(Layer):
         confidence = confidence.permute(0, 2, 3, 1).contiguous()
         confidence = confidence.view(confidence.size(0), -1, self.num_classes)
 
-        if 'softmax' not in self.__dict__:
-            self.softmax = SoftMax(-1)
-        confidence = self.softmax(confidence)
-
         location = self.regression_headers[i](x)
         location = location.permute(0, 2, 3, 1).contiguous()
-
-        num_batches = location.size(0)
-        location = location.view(num_batches, -1, self.num_regressors)
-
+        location = location.view(location.size(0), -1, self.num_regressors)
         return confidence, location
 
     def area_of(self, left_top, right_bottom) -> torch.Tensor:
@@ -418,7 +411,7 @@ class RFBnet(Layer):
             subset_boxes = boxes[mask, :]
             box_probs = concate([subset_boxes, probs.reshape(-1, 1)], axis=1)
             box_probs = self.hard_nms(box_probs,
-                                      iou_threshold=iou_threshold,
+                                      nms_threshold=iou_threshold,
                                       top_k=top_k,
                                       )
             picked_box_probs.append(box_probs)
@@ -477,12 +470,16 @@ class RFBnet(Layer):
         locations.append(location3)
 
         confidences = torch.cat(confidences, 1)
+        if not hasattr(self,'softmax') or self.softmax is None:
+            self.softmax=SoftMax(-1)
+        confidences=self.softmax(confidences)
+
         locations = torch.cat(locations, 1)
 
         if self.training:
             result = OrderedDict()
             result['confidences'] = confidences
-            result['locations'] = locations.view(locations.size(0), -1, self.num_regressors)
+            result['locations'] = locations
             return result
 
         else:
