@@ -111,11 +111,15 @@ def resblock_body(num_filters, num_blocks, all_narrow=True,keep_output=False,nam
         DarknetConv2D_BN_Mish((1,1),num_filters,name=name+'_convblock5')
         )
 
+anchors1 = to_tensor(np.array([12, 16, 19, 36, 40, 28]).reshape(-1, 2),requires_grad=False)
+anchors2 = to_tensor(np.array([36, 75, 76, 55, 72, 146]).reshape(-1, 2),requires_grad=False)
+anchors3 = to_tensor(np.array([142, 110, 192, 243, 459, 401]).reshape(-1, 2),requires_grad=False)
+anchors=(anchors1,anchors2,anchors3)
 
-def yolo4_body(anchors=None,num_classes=80,image_size=608):
-    anchors1=generate_anchors(76) if anchors is None else anchors[0]
-    anchors2 = generate_anchors(38) if anchors is None else anchors[1]
-    anchors3 = generate_anchors(19) if anchors is None else anchors[2]
+def yolo4_body(num_classes=80,image_size=608,anchors=anchors):
+    anchors1=anchors[0]
+    anchors2 = anchors[1]
+    anchors3 = anchors[2]
     num_anchors=len(anchors1)
     """Create YOLO_V4 model CNN body in Pytorch."""
     return Sequential(
@@ -148,7 +152,7 @@ def yolo4_body(anchors=None,num_classes=80,image_size=608):
                                     Upsampling2d(scale_factor=2,name='y19_upsample'),
                                 ),
                                 0:DarknetConv2D_BN_Leaky((1, 1), 256)
-                            },mode='concate'),
+                            },mode='concate',axis=1),
                         DarknetConv2D_BN_Leaky((1, 1),256,name='pre_y38_1'),
                         DarknetConv2D_BN_Leaky((3, 3),512,name='pre_y38_2'),
                         DarknetConv2D_BN_Leaky((1, 1),256,name='pre_y38_3'),
@@ -159,7 +163,7 @@ def yolo4_body(anchors=None,num_classes=80,image_size=608):
                     ),
                     0:DarknetConv2D_BN_Leaky((1, 1), 128)
                 },
-                mode='concate'),
+                mode='concate',axis=1),
             DarknetConv2D_BN_Leaky((1, 1), 128,name='pre_y76_concate1'),
             DarknetConv2D_BN_Leaky((3, 3), 256,name='pre_y76_concate2'),
             DarknetConv2D_BN_Leaky((1, 1), 128,name='pre_y76_concate3'),
@@ -176,7 +180,7 @@ def yolo4_body(anchors=None,num_classes=80,image_size=608):
                 Sequential(
                     ShortCut2d(
                         DarknetConv2D_BN_Leaky((3, 3), 256, strides=(2, 2), auto_pad=False,padding=((1,0),(1,0)),name='y76_downsample'),
-                        branch_from='y_38',mode='concate'),
+                        branch_from='y_38',mode='concate',axis=1),
                     DarknetConv2D_BN_Leaky((1, 1), 256,name='pre_y38_concate1'),
                     DarknetConv2D_BN_Leaky((3, 3), 512,name='pre_y38_concate2'),
                     DarknetConv2D_BN_Leaky((1, 1), 256,name='pre_y38_concate3'),
@@ -192,7 +196,7 @@ def yolo4_body(anchors=None,num_classes=80,image_size=608):
                         Sequential(
                             ShortCut2d(
                                 DarknetConv2D_BN_Leaky((3, 3), 512, strides=(2, 2),auto_pad=False,padding=((1,0),(1,0)),name='y38_downsample'),
-                                branch_from='y_19', mode='concate'),
+                                branch_from='y_19', mode='concate',axis=1),
                             DarknetConv2D_BN_Leaky((1, 1), 512,name='pre_y19_concate1'),
                             DarknetConv2D_BN_Leaky((3, 3), 1024,name='pre_y19_concate2'),
                             DarknetConv2D_BN_Leaky((1, 1), 512,name='pre_y19_concate3'),
@@ -204,9 +208,9 @@ def yolo4_body(anchors=None,num_classes=80,image_size=608):
                                 YoloLayer(anchors=anchors3,num_classes=num_classes,grid_size=19, img_dim=image_size),
                             name='y19_output')),
 
-                        mode='concate')
+                        mode='concate',axis=1)
                 )
-                ,mode = 'concate')
+                ,mode = 'concate',axis=1)
     )
 
 
