@@ -24,12 +24,12 @@ from trident.layers.pytorch_layers import Dense
 
 from trident import context
 from trident.backend.model import ModelBase
-from trident.backend.common import dtype as Dtype
+from trident.backend import dtype as Dtype
 from trident.backend.common import *
 from trident.backend.pytorch_backend import *
 from trident.backend.pytorch_ops import *
 from trident.layers.pytorch_activations import sigmoid
-from trident.optims.losses import Loss
+from trident.optims.losses import Loss,_check_logsoftmax_logit
 
 # from trident.optims.pytorch_trainer import Model
 
@@ -124,7 +124,7 @@ class _ClassificationLoss(Loss):
 
                 elif ds is not None and ds.__class__.__name__ == 'MaskDataset' and dp.traindata.label.object_type in [ObjectType.label_mask, ObjectType.color_mask]:
                     print('Start retrive label class distribution for auto-balance in loss function.')
-                    unique, counts = torch.unique(to_tensor(np.stack([dp.traindata.label[i] for i in tqdm(range(len(dp.traindata.label)))]), dtype=dtype.long, device='cpu'),
+                    unique, counts = torch.unique(to_tensor(np.stack([dp.traindata.label[i] for i in tqdm(range(len(dp.traindata.label)))]), dtype=Dtype.long, device='cpu'),
                                                   return_counts=True)
                     unique = to_list(to_numpy(unique))
                     counts = to_numpy(counts)
@@ -149,30 +149,31 @@ class _ClassificationLoss(Loss):
         ndim_target = ndim(target)
         if ndim(output) > 2:
             if self.axis == 1:
-                output = reshape(output, (output.size(0), output.size(1), -1))
-
-
+                output=output.view(output.size(0), output.size(1), -1)
             elif self.axis == -1:
-                output = reshape(output, (output.size(0), -1, output.size(-1)))
+                output=output.view(output.size(0), -1,output.size(-1))
+                output=output.transpose(2,1).continuous()
 
-            if ndim_target == ndim_output - 1 and target.dtype == dtype.long:
-                target = reshape(target, (target.size(0), -1))
+            if ndim_target == ndim_output - 1 and target.dtype == Dtype.long:
+                if ndim_target>=2:
+                    target=target.view(target.size(0),  -1)
 
-            elif ndim_target == ndim_output and target.dtype != dtype.long:
+
+            elif ndim_target == ndim_output and target.dtype != Dtype.long:
                 if self.axis == 1:
-                    target = reshape(target, (target.size(0), target.size(1), -1))
-
-
+                    target=target.view(target.size(0), target.size(1), -1)
                 elif self.axis == -1:
-                    target = reshape(target, (-1, target.size(-1)))
+                    target=target.view(target.size(0), -1,target.size(-1))
+                    target = target.transpose(2, 1).continuous()
 
-            return output.contiguous(), target
-        elif ndim(output) <= 2 and len(output) == len(target):
             return output, target
-        elif ndim(output) <= 2 and ndim(output) == ndim(target) + 1:
+        elif ndim(output) <= 2:
+
             return output, target
         else:
-            raise ValueError('output and target have diffent elements.')
+
+            return output, target
+            #raise ValueError('output and target have diffent elements.')
 
     def preprocess(self, output: Tensor, target: Tensor, **kwargs):
         """
@@ -185,6 +186,7 @@ class _ClassificationLoss(Loss):
         Returns:
 
         """
+        #print(2, 'output', output.shape, 'abnormal:', any_abnormal_number(output))
         # check num_clases
         if len(output) == 0:
             return to_tensor(0.0)
@@ -193,49 +195,38 @@ class _ClassificationLoss(Loss):
 
         if self.sample_weight is None:
             self.sample_weight = ones(self.num_classes, requires_grad=False).to(get_device())
-        elif len(self.sample_weight) != self.num_classes:
+        else:
+            self.sample_weight=to_tensor(self.sample_weight ).detach().to(output.device)
+        if len(self.sample_weight) != self.num_classes:
             raise ValueError('weight should be 1-D tensor and length equal to numbers of filters')
-        elif self.sample_weight.requires_grad != False or self.sample_weight.dtype != output.dtype or self.sample_weight.device != output.device:
-            self.sample_weight = to_tensor(self.sample_weight, requires_grad=False).to(get_device())
-        else:
-            pass
 
-        output_exp = exp(output)
-        if ndim(output) >= 1 and 'float' in str(output.dtype) and reduce_min(output) >= 0 :
-            self.is_logsoftmax = False
-            if reduce_max(output) > 1:
-                output=output/reduce_max(output)
 
-            sum_output=reduce_sum(output,axis=self.axis,keepdims=True)
-            if reduce_max(abs(sum_output-1))<1e-3:
+        if not self.is_logsoftmax:
+            if reduce_min(output) >= 0 :
+                self.is_logsoftmax = False
+                sum_output=reduce_sum(output,axis=self.axis,keepdims=True)
+                if reduce_max(output) > 1:
+                    self.from_logits = False
+                elif reduce_max(abs(sum_output-1))<1e-2:
+                    self.from_logits = True
+                else:
+                    self.from_logits = False
+
+                output = clip(output, min=1e-7, max=1)
+
+            elif  _check_logsoftmax_logit(output,self.axis):
+                self.is_logsoftmax = True
                 self.from_logits = True
+                output = clip(output,min=-12 ,max=-1e-7)
+
             else:
-                output=output/clip(sum_output,1e-7,1-1e-7)
-                self.from_logits = True
+                self.is_logsoftmax = False
+                self.from_logits = False
 
-            if self.auto_balance and self.label_statistics is not None:
-                if self.num_classes== len(self.label_statistics):
-                    new_shp = [1] * len(int_shape(output))
-                    new_shp[1] = len(self.label_statistics)
-                    output = output * clip(to_tensor(np.reshape(self.label_statistics.copy(), tuple(new_shp)),requires_grad=False), min=1e-7)
-
-            output = clip(output, min=1e-7, max=1)
-
-        elif (ndim(output) >= 1 and 'float' in str(output.dtype) and output_exp.min() >= 0 and output_exp.max() <= 1):
-            self.is_logsoftmax = True
-            self.from_logits = True
-            if self.auto_balance and self.label_statistics is not None:
-                if int_shape(output)[1] == len(self.label_statistics):
-                    new_shp = [1] * len(int_shape(output))
-                    new_shp[1] = len(self.label_statistics)
-                    output = output - to_tensor(np.reshape(np.log(clip(self.label_statistics.copy(), min=1e-7)), tuple(new_shp)))
-            output = clip(output, max=- 1e-7)
+            if (ndim(target) == ndim(output) and 'float' in str(target.dtype) and target.min() >= 0 and target.max() <= 1):
+                self.is_target_onehot = True
         else:
-            self.is_logsoftmax = False
-            self.from_logits = False
-
-        if (ndim(target) == ndim(output) and 'float' in str(target.dtype) and target.min() >= 0 and target.max() <= 1):
-            self.is_target_onehot = True
+            output = clip(output, min=-12, max=-1e-7)
 
         self.ignore_index_weight = ones_like(self.sample_weight, requires_grad=False, dtype=output.dtype).to(get_device())
         # ignore_index
@@ -251,11 +242,11 @@ class _ClassificationLoss(Loss):
         if target.dtype == str2dtype('long'):
             self.is_target_onehot = False
         elif target.dtype != str2dtype('long') and (target.min() >= 0 and target.max() <= 1 and abs(target.sum(self.axis)-1).mean() < 1e-4):
-            target = clip(target, min=1e-7, max=1- 1e-7)
+            #target = clip(target, min=1e-7, max=1- 1e-7)
             self.is_target_onehot = True
 
         # need target onehot but currently not
-        if target.dtype == torch.long and self.need_target_onehot == True and self.is_target_onehot == False:
+        if target.dtype == Dtype.long and self.need_target_onehot == True and self.is_target_onehot == False:
             target = make_onehot(target, num_classes=self.num_classes, axis=self.axis).to(get_device())
 
             target.require_grads = False
@@ -272,6 +263,8 @@ class _ClassificationLoss(Loss):
         # if self.cutoff is not None:
         #     mask = (output > self.cutoff).to(output.dtype)
         #     output = output * mask
+        #print(8, 'output', output.shape, 'abnormal:', any_abnormal_number(output))
+
         return output, target
 
     calculate_loss: Callable[..., Any] = _calculate_loss_unimplemented
@@ -290,7 +283,7 @@ class _ClassificationLoss(Loss):
     def _handel_abnormal(self, loss):
         if any_abnormal_number(loss):
             sys.stderr.write('{0} has abnormal value,trident automatically replace these abnormal value to zero.\n'.format(self.name))
-            loss = where(is_abnormal_number(loss), ones_like(loss, requires_grad=False) * 1e-7, loss)
+            loss = where(is_abnormal_number(loss), zeros_like(loss) , loss)
         return loss
 
     def forward(self, output: Tensor, target: Tensor, **kwargs) -> 'loss':
@@ -305,12 +298,15 @@ class _ClassificationLoss(Loss):
 
             """
         try:
-
+            #print(0,'output',output.shape,'abnormal:',any_abnormal_number(output))
             output, target = self.flatten_check(output, target)
+            #print(1, 'output', output.shape, 'abnormal:', any_abnormal_number(output))
             if self.auto_balance and self.label_statistics is None:
                 self._calculate_label_statistics()
             loss = self.calculate_loss(*self.preprocess(output, target.detach(), **kwargs))
+
             loss = self._handel_abnormal(loss)
+
             loss = self._get_reduction(loss)
             return loss
         except Exception as e:
@@ -418,7 +414,7 @@ class _PairwiseLoss(Loss):
             if self.enable_ohem:
                 output, target = self._do_ohem(output, target)
             return output, target
-        elif target.dtype == torch.int64 and ndim(output) == ndim(target) + 1:
+        elif target.dtype == Dtype.int64 and ndim(output) == ndim(target) + 1:
             num_class = int_shape(output)[self.axis]
             target = make_onehot(target, num_class, self.axis).float()
             if self.enable_ohem:
@@ -595,31 +591,53 @@ class CrossEntropyLoss(_ClassificationLoss):
 
         Returns:
 
+        Examples:
+        >>> output =  to_tensor([[0.3, 0.3, 0.2], [0, 1, 0.5], [1, 1, 0]],dtype=torch.float32)
+        >>> target = to_tensor([[0, 1, 0], [0, 1, 0], [1, 0, 0]],dtype=torch.float32)
+        >>> CrossEntropyLoss(reduction='mean')(output,target).cpu()
+        tensor(0.8695)
+        >>> CrossEntropyLoss(reduction='mean')(output,argmax(target,-1)).cpu()
+        tensor(0.8695)
+        >>> CrossEntropyLoss(reduction='mean')(log_softmax(output),target).cpu()
+        tensor(0.8695)
+        >>> CrossEntropyLoss(reduction='mean')(log_softmax(output),argmax(target,-1)).cpu()
+        tensor(0.8695)
+        >>> CrossEntropyLoss(reduction='mean', sample_weight=to_tensor([1,2,3]))(output,target).cpu()
+        tensor(1.4518)
         """
-        loss = to_tensor(0.0)
 
-        sample_weight = cast(self.sample_weight, output.dtype) * cast(self.ignore_index_weight, output.dtype)
+        sample_weight = (cast(self.sample_weight, output.dtype) * cast(self.ignore_index_weight, output.dtype)).detach()
+
+
+
         if not self.need_target_onehot:
-            if self.is_target_onehot and target.dtype != dtype.long:
+            if self.is_target_onehot and target.dtype != Dtype.long:
                 target = argmax(target, self.axis)
-            if not self.is_logsoftmax:
-                return torch.nn.functional.nll_loss(torch.nn.functional.log_softmax(output, dim=1), target, weight=sample_weight, reduction='none')
-            else:
-                return torch.nn.functional.nll_loss(output, target, weight=sample_weight, reduction='none')
+            #print(9, 'output', output.shape, 'abnormal:', any_abnormal_number(output))
+
+            unbalance_weight = zeros(tuple([1]*(ndim(target)-1)+[ self.num_classes]))
+            if self.auto_balance and self.label_statistics is not None:
+                if int_shape(output)[self.axis] == len(self.label_statistics):
+                    unbalance_weight =to_tensor(np.log(clip(self.label_statistics.copy(), min=1e-7)))
+
+            target_shape=target.shape
+            gather_unbalance_weight=torch.index_select(unbalance_weight, -1, target.reshape((-1))).reshape(target_shape).detach()
+            #print(10, 'output', output.shape, 'abnormal:', any_abnormal_number(output))
+            if not self.is_logsoftmax and reduce_min(output)>=0:
+                output=clip(log_softmax(output, axis=1), min=-12, max=-1e-7)
+            target = target.detach()
+            return nn.functional.nll_loss(output,target,sample_weight,reduction='none')+gather_unbalance_weight
         else:
+            unbalance_weight=ones(tuple([1]*(ndim(output)-1)+[ self.num_classes]))
+            if self.auto_balance and self.label_statistics is not None:
+                if self.num_classes== len(self.label_statistics):
+                    unbalance_weight= clip(to_tensor(np.reshape(self.label_statistics.copy(), tuple([1]*(ndim(output)-1)+[ self.num_classes])),requires_grad=False), min=1e-7)
 
-            if ndim(output) == 2:
-                if self.is_logsoftmax:
-                    sample_weight = self.sample_weight
-            else:
-                reshape_shape = [1] * ndim(output)
-                reshape_shape[self.axis] = self.num_classes
-                sample_weight = self.sample_weight.view(*reshape_shape) * self.ignore_index_weight.view(*reshape_shape)
-
-            if not self.is_logsoftmax:
-                return -reduce_sum(target * F.log_softmax(output, dim=self.axis) * sample_weight, axis=self.axis)
-            else:
-                return -reduce_sum(target * output * sample_weight, axis=self.axis)
+            if not self.is_logsoftmax and reduce_min(output)>=0:
+                output=clip(log_softmax(output, axis=1), min=-12, max=-1e-7)
+            sample_weight = expand_as(sample_weight, output)
+            target=target.detach()
+            return -reduce_sum(target * output * sample_weight*unbalance_weight, axis=1)
 
 
 
@@ -823,23 +841,28 @@ class F1ScoreLoss(_ClassificationLoss):
         self._built = True
 
     def calculate_loss(self, output, target, **kwargs):
+        sample_weight=(self.sample_weight * self.ignore_index_weight).detach()
+        n_ = ndim(target) - ndim(sample_weight)
+        for n in range(n_):
+            sample_weight = expand_dims(sample_weight, 0)
 
         if self.is_logsoftmax:
-            output = clip(exp(output), 1e-8, 1 - 1e-8)
-            self.from_logits = True
-        if not self.from_logits:
-            output = softmax(output, self.axis)
-        if target.dtype == torch.int64 or self.is_target_onehot == False:
-            target = make_onehot(target, self.num_classes, axis=1).to(output.dtype)
-        target.require_grads = False
+            output =clip(exp(clip(output,max=0)),1e-7,1-1e-7)
 
-        tp = (target * output * self.sample_weight * self.ignore_index_weight).sum(axis=self.axis)
+        elif not self.from_logits:
+            output = softmax(output, self.axis)
+        if target.dtype == Dtype.int64 or self.is_target_onehot == False:
+            target = make_onehot(target, self.num_classes, axis=self.axis).to(output.dtype)
+        target.detach()
+
+
+        tp = reduce_sum(target * output*sample_weight ,axis=self.axis)
         tn = ((1 - target) * (1 - output))
         fp = ((1 - target) * output)
         fn = (target * (1 - output))
-        precision = true_divide(tp, reduce_sum(output, axis=self.axis))
-        recall = true_divide(tp, reduce_sum(target, axis=self.axis))
-        return 1 - (1 + self.beta ** 2) * precision * recall / (self.beta ** 2 * precision + recall)
+        precision = true_divide(tp, reduce_sum(output ,axis=self.axis))
+        recall = true_divide(tp,reduce_sum(target ,axis=self.axis))
+        return 1 - (1 + self.beta ** 2) * precision * recall/  (self.beta ** 2 * precision + recall)
 
 
 class FocalLoss(_ClassificationLoss):
@@ -898,33 +921,40 @@ class FocalLoss(_ClassificationLoss):
         Returns:
 
             """
+
+        sample_weight = cast(self.sample_weight, output.dtype) * cast(self.ignore_index_weight, output.dtype)
+        sample_weight = sample_weight.reshape(tuple([1] * (ndim(output) - 1) + [self.num_classes])).detach()
         alpha = torch.zeros(self.num_classes).to(output.device)
         alpha[0] += self.alpha
         alpha[1:] += (1 - self.alpha)
-        if ndim(output) > 2 and self.axis==1:
-            output = output.view(output.size(0), output.size(1), -1)  # N,C,H,W => N,C,H*W
+        if ndim(output) > 2 and self.axis == 1:
             output = output.transpose(1, 2)  # N,C,H*W => N,H*W,C
             output = output.contiguous().view(-1, output.size(2))  # N,H*W,C => N*H*W,C
 
-
-        if self.is_target_onehot and target.dtype != dtype.long:
-            target = argmax(target, self.axis)
-            self.is_target_onehot=False
-        target = target.view(-1, 1)
-
         if not self.is_logsoftmax:
             output=log_softmax(output)
-            self.is_logsoftmax=True
+            output = clip(output, min=-12, max=-1e-7)
 
-        logpt=output
+        if self.is_target_onehot and target.dtype != Dtype.long:
+            target = argmax(target, self.axis)
+        target = target.view(-1, 1)
+
+        unbalance_weight = zeros(tuple([1] * (ndim(target) - 1) + [self.num_classes]))
+        if self.auto_balance and self.label_statistics is not None:
+            if int_shape(output)[self.axis] == len(self.label_statistics):
+                unbalance_weight = to_tensor(np.log(clip(self.label_statistics.copy(), min=1e-7)))
+
+        target_shape = target.shape
+        gather_unbalance_weight = torch.index_select(unbalance_weight, -1, target.reshape((-1))).reshape(target_shape).detach()
+
+        logpt = output
         logpt = logpt.gather(1, target)
         pt = logpt.data.exp()
-        #
-        loss=-1 *  ((1 - pt) ** self.gamma) * logpt
-        alpha=alpha.gather(0, target.view(-1))
-        loss=loss*(alpha.view(-1,1))
-        return loss
 
+        loss = -1 * ((1 - pt) ** self.gamma) * logpt
+        alpha = alpha.gather(0, target.view(-1))
+        loss = loss * (alpha.view(-1, 1))
+        return loss
 
 
         # - \alpha(1 - softmax(x)[class ]) ^ gamma \log(softmax(x)[class])
@@ -976,7 +1006,7 @@ class BCELoss(_ClassificationLoss):
 
         """
         if self.is_logsoftmax:
-            output = exp(output)
+            output = clip(exp(clip(output,max=0)),1e-7,1-1e-7)
 
         loss = binary_cross_entropy(output, target, from_logits=self.from_logits)
         return loss
@@ -1056,22 +1086,36 @@ class DiceLoss(_ClassificationLoss):
         Returns:
 
         """
+        # if any_abnormal_number(output):
+        #     print('{0} calculate starting'.format(self.name))
         if self.is_logsoftmax:
-            output = exp(output)
-        reduce_axes = list(range(target.ndim))
-        axis = self.axis if self.axis >= 0 else target.ndim + self.axis
-        reduce_axes.remove(0)
-        sample_weight = expand_dims(self.sample_weight.to(get_device()) * self.ignore_index_weight.to(get_device()), 0)
-        n_ = ndim(output) - ndim(sample_weight)
-        for n in range(n_):
-            sample_weight = expand_dims(sample_weight, -1)
+            output =clip(exp(clip(output,max=0)),1e-7,1-1e-7)
+        #
+        # if any_abnormal_number(output):
+        #     print('{0} after exp'.format(self.name))
+        target=target.detach()
+        sample_weight = (self.sample_weight.to(get_device()) * self.ignore_index_weight.to(get_device())).detach()
+
+        intersection = reduce_sum(target * output, axis=-1)
+        # if any_abnormal_number(intersection):
+        #     print('{0} after intersection'.format(self.name))
+        den1 = reduce_sum(output, axis=-1)
+        # if any_abnormal_number(den1):
+        #     print('{0} after den1'.format(self.name))
+        den2 = reduce_sum(target, axis=-1)
+        # if any_abnormal_number(den2):
+        #     print('{0} after den2'.format(self.name))
+
+        sample_weight = expand_as(sample_weight, intersection).detach()
+
+        if output.shape[1]==2:
+            dice = (1.0 - (2.0 * (intersection * sample_weight)[:,1:].sum(1) + self.smooth) / (den1[:,1:].sum(1) + den2[:,1:].sum(1) + self.smooth))
+        else:
+            dice = (1.0 - (2.0 * (intersection * sample_weight).sum(1) + self.smooth) / (den1.sum(1) + den2.sum(1) + self.smooth))
+
+        # if any_abnormal_number(den2):
+        #     print('{0} after dice {1}'.format(self.name,dice))
 
-        # for k in range(target.ndim-self.loss_weights.ndim):
-        #     loss_weights=loss_weights.expand_dims(0)
-        intersection = reduce_sum(target * output * sample_weight, axis=reduce_axes)
-        den1 = reduce_sum(output * sample_weight, axis=reduce_axes)
-        den2 = reduce_sum(target * sample_weight, axis=reduce_axes)
-        dice = 1.0 - (2.0 * intersection + self.smooth) / (den1 + den2 + self.smooth)
         return dice
 
 
@@ -1154,7 +1198,7 @@ class L2Loss(_PairwiseLoss):
 
         """
         batch = int_shape(output)[0]
-        return 0.5 * F.mse_loss(output.view(batch, -1), target.view(batch, -1), reduction='none')
+        return 0.5 * ((output.view(batch, -1)-target.view(batch, -1))**2)
 
 
 class SmoothL1Loss(_PairwiseLoss):
@@ -1210,7 +1254,7 @@ class MSELoss(_PairwiseLoss):
 
         """
         batch = int_shape(output)[0]
-        return F.mse_loss(output.view(batch, -1), target.view(batch, -1), reduction='none')
+        return ((output.view(batch, -1)-target.view(batch, -1))**2)
 
 
 class WingLoss(_PairwiseLoss):
@@ -1359,7 +1403,7 @@ def create_window(window_size: int, sigma: float, channel: int):
     :param channel: input channel
     :return: 1D kernel
     '''
-    coords = torch.arange(window_size, dtype=torch.float)
+    coords = torch.arange(window_size, dtype=Dtype.float)
     coords -= window_size // 2
 
     g = torch.exp(-(coords ** 2) / (2 * sigma ** 2))
@@ -1401,9 +1445,12 @@ def ssim(X, Y, window, data_range: float, use_padding: bool = False):
     K1 = 0.01
     K2 = 0.03
     compensation = 1.0
-
-    C1 = (K1 * data_range) ** 2
-    C2 = (K2 * data_range) ** 2
+    if data_range==2:
+        C1 = ((K1 * data_range)/data_range) ** 2
+        C2 = ((K2 * data_range)/data_range) ** 2
+    else:
+        C1 = (K1 * data_range) ** 2
+        C2 = (K2 * data_range) ** 2
 
     mu1 = _gaussian_filter(X, window, use_padding)
     mu2 = _gaussian_filter(Y, window, use_padding)
@@ -1424,15 +1471,15 @@ def ssim(X, Y, window, data_range: float, use_padding: bool = False):
     cs_map = F.relu(cs_map)
     ssim_map = ((2 * mu1_mu2 + C1) / (mu1_sq + mu2_sq + C1)) * cs_map
 
-    ssim_val = ssim_map.mean(dim=(1, 2, 3))  # reduce along CHW
-    cs = cs_map.mean(dim=(1, 2, 3))
+    ssim_val = ssim_map.sum(dim=1,keepdim=True).mean(dim=(2, 3)).squeeze()  # reduce along CHW
+    cs = cs_map.sum(dim=1,keepdim=True).mean(dim=(2, 3)).squeeze()
 
     return ssim_val, cs
 
 
 class MS_SSIMLoss(_PairwiseLoss):
-    def __init__(self, window_size=11, window_sigma=1.5, data_range=255., channel=3, use_padding=False, weights=None, levels=4, eps=1e-8,input_names=None, output_names=None,name='ms_ssim'):
-        super(MS_SSIMLoss, self).__init__()
+    def __init__(self, window_size=11, window_sigma=1.5, data_range=255., channel=3, use_padding=False, weights=None, levels=4, eps=1e-8,reduction='mean', enable_ohem=False, ohem_ratio=3.5,input_names=None, output_names=None,name='ms_ssim'):
+        super(MS_SSIMLoss, self).__init__(reduction='mean', enable_ohem=False, ohem_ratio=3.5,input_names=input_names, output_names=output_names, name=name)
 
         self.window_size = window_size
         self.channel = channel
@@ -1445,7 +1492,7 @@ class MS_SSIMLoss(_PairwiseLoss):
 
         if weights is None:
             weights = [0.0448, 0.2856, 0.3001, 0.2363, 0.1333]
-        weights = torch.tensor(weights, dtype=torch.float, requires_grad=False)
+        weights = torch.tensor(weights, dtype=Dtype.float, requires_grad=False)
         self.levels = levels
         if levels is not None:
             weights = weights[:levels]
@@ -1503,30 +1550,25 @@ class IoULoss(_ClassificationLoss):
     def __init__(self, axis=1, sample_weight=None, auto_balance=False, from_logits=False, ignore_index=-100, cutoff=None, label_smooth=False, reduction='mean', enable_ohem=False,
                  ohem_ratio=3.5,  input_names=None, output_names=None,name='lou_loss'):
         super(IoULoss, self).__init__(raxis=axis, sample_weight=sample_weight, auto_balance=auto_balance, from_logits=from_logits, ignore_index=ignore_index, cutoff=cutoff,
-                                      label_smooth=label_smooth, reduction=reduction, enable_ohem=enable_ohem, ohem_ratio=ohem_ratio, name=name)
-        self.need_target_onehot = False
+                                      label_smooth=label_smooth, reduction=reduction, enable_ohem=enable_ohem, ohem_ratio=ohem_ratio,input_names=input_names ,output_names=output_names,name=name)
+        self.need_target_onehot = True
         self.is_multiselection = False
         self.is_logsoftmax = False
         self._built = True
 
     def calculate_loss(self, output, target, **kwargs):
-        _dtype = output.dtype
         if self.is_logsoftmax:
-            output = clip(exp(output), 1e-8, 1 - 1e-8)
-            self.from_logits = True
-        if not self.from_logits:
-            output = softmax(output, self.axis)
-        if target.dtype != torch.int64 or self.is_target_onehot == True:
-            target = argmax(target, axis=self.axis)
-        batch_size = int_shape(output)[0]
-        output = argmax(output, axis=self.axis)
+            output =clip(exp(clip(output,max=0)),1e-7,1-1e-7)
 
-        output_flat = output.reshape((batch_size, -1))
-        target_flat = target.reshape((batch_size, -1))
-        intersection = equal(output_flat, target_flat, dtype=_dtype).sum()
-        union = greater(greater(output_flat, 0, dtype=_dtype) + greater(target_flat, 0, dtype=_dtype), 0, dtype=_dtype).sum().clamp(min=1)
-        loss = 1 - (intersection / union)
-        return loss
+        sample_weight = (self.sample_weight.to(get_device()) * self.ignore_index_weight.to(get_device()))
+        target=target.detach()
+        sample_weight = expand_as(sample_weight, output).detach()
+        intersection = reduce_sum(target * output*sample_weight, axis=-1)
+        union = reduce_sum(output*sample_weight, axis=-1)+reduce_sum(target, axis=-1)-intersection
+
+
+        iou=( intersection).sum(1) / (union + 1e-7).sum(1)
+        return (1-iou).mean()
 
 
 class SoftIoULoss(Loss):
@@ -2014,8 +2056,6 @@ class EdgeLoss(_Loss):
         super(EdgeLoss, self).__init__()
 
         self.reduction = reduction
-
-        self.styleloss = StyleLoss()
         self.to(_device)
 
     def first_order(self, x, axis=2):
