@@ -62,7 +62,7 @@ class Optimizer(optimizer.Optimizer):
         Returns: the weights need to train
 
         """
-        return self.param_groups[0]['params']
+        return [self.param_groups[i]['params'] for i in range(len(self.param_groups))]
 
     @parameters.setter
     def parameters(self,value):
@@ -152,7 +152,9 @@ class Adam(Optimizer):
             raise ValueError("Invalid weight_decay value: {}".format(weight_decay))
         defaults = dict(lr=lr, betas=betas, eps=eps,
                         weight_decay=weight_decay, amsgrad=amsgrad)
+
         super(Adam, self).__init__(params, defaults)
+        self.gradient_centralization=gradient_centralization
 
     def __setstate__(self, state):
         super(Adam, self).__setstate__(state)
@@ -217,6 +219,7 @@ class Adam(Optimizer):
 
                 if group['weight_decay'] != 0:
                     grad = grad.add(p, alpha=group['weight_decay'])
+
                 if self.gradient_centralization in ['all', 'gcc']:
                     if len(list(grad.size())) > 3:
                         grad.add_(-grad.mean(dim=tuple(range(1, grad.dim())), keepdim=True))
@@ -238,8 +241,8 @@ class Adam(Optimizer):
 
                 G_grad = exp_avg / denom
                 if self.gradient_centralization in ['all', 'gc']:
-                    if len(list(G_grad.size())) > 1:
-                        G_grad.add_(-G_grad.mean(dim=tuple(range(1, len(list(G_grad.size())))), keepdim=True))
+                    if ndim(G_grad) > 1:
+                        G_grad.add_(-G_grad.mean(axis=list(range(1, ndim(G_grad))), keepdims=True))
 
                 p.data.add_(G_grad, alpha=-step_size)
                 if half_precision:
@@ -1252,15 +1255,17 @@ class Ranger(Optimizer):
                 if group['weight_decay'] != 0:
                     G_grad.add_(p_data_fp32, alpha=group['weight_decay'])
 
+
                 if self.gradient_centralization in ['all', 'gc']:
                     if len(list(G_grad.size())) > 1:
                         G_grad.add_(-G_grad.mean(dim=tuple(range(1, len(list(G_grad.size())))), keepdim=True))
 
+                p_data_fp32.add_(G_grad, alpha=-step_size * group['lr'])
                 if any_abnormal_number(p_data_fp32):
                     sys.stderr.write('{0} p_data has abnormal value,trident automatically replace these abnormal value to zero.\n\r'.format(self.__class__.__name__))
                     p_data_fp32 = where(is_abnormal_number(p_data_fp32),p.data.float(), p_data_fp32)
 
-                p_data_fp32.add_(G_grad, alpha=-step_size * group['lr'])
+
                 p.data.copy_(p_data_fp32)
 
                 if state['step'] % group['k'] == 0:
@@ -1406,18 +1411,20 @@ class RangerLars(Optimizer):
                 update = zeros_like(p_data)
                 if N_sma >= 5:
                     denom = exp_avg_sq.sqrt().add_(group['eps'])
-                    update.addcdiv_(exp_avg, denom, value=step_size)
+
+                    G_grad = (exp_avg/denom) * step_size
                 else:
-                    update.add_(exp_avg, alpha=step_size)
+                    G_grad=exp_avg*step_size
+
 
                 # if group['weight_decay'] != 0:
                 #     update.add_(group['weight_decay'], p_data)
                 if group['weight_decay'] != 0:
-                    grad = grad.add(p, alpha=group['weight_decay'])
+                    G_grad = G_grad.add(p, alpha=group['weight_decay'])
 
                 if self.gradient_centralization in ['all', 'gc']:
-                    if len(list(grad.size())) > 1:
-                        grad.add_(-grad.mean(dim=tuple(range(1, len(list(grad.size())))), keepdim=True))
+                    if ndim(G_grad)> 1:
+                        G_grad=G_grad-G_grad.mean(axis=list(range(1, ndim(G_grad))), keepdims=True)
 
                 radam_norm = update.pow(2.0).sum().sqrt()
                 weight_norm = p.data.pow(2.0).sum().sqrt()
@@ -1650,7 +1657,7 @@ class AdaBelief(Optimizer):
     reference: AdaBelief Optimizer, adapting stepsizes by the belief in observed gradients, NeurIPS 2020
     """
 
-    def __init__(self, params, lr=1e-3, betas=(0.9, 0.999), eps=1e-16,
+    def __init__(self, params, lr=1e-3, betas=(0.9, 0.999), eps=1e-7,
                  weight_decay=0, amsgrad=False, weight_decouple=True, fixed_decay=False, rectify=True,
                  degenerated_to_sgd=True, gradient_centralization=None,**kwargs):
         if not 0.0 <= lr:
@@ -1662,7 +1669,6 @@ class AdaBelief(Optimizer):
         if not 0.0 <= betas[1] < 1.0:
             raise ValueError("Invalid beta parameter at index 1: {}".format(betas[1]))
 
-        self.degenerated_to_sgd = degenerated_to_sgd
         if isinstance(params, (list, tuple)) and len(params) > 0 and isinstance(params[0], dict):
             for param in params:
                 if 'betas' in param and (param['betas'][0] != betas[0] or param['betas'][1] != betas[1]):
@@ -1765,8 +1771,8 @@ class AdaBelief(Optimizer):
                 exp_avg, exp_avg_sq = state['exp_avg'], state['exp_avg_sq']
 
                 if self.gradient_centralization in ['all', 'gcc']:
-                    if len(list(grad.size())) > 3:
-                        grad.add_(-grad.mean(dim=tuple(range(1, grad.dim())), keepdim=True))
+                    if ndim(grad)> 3:
+                        grad=grad-grad.mean(axis=list(range(1, ndim(grad))), keepdims=True)
 
                 state['step'] += 1
                 bias_correction1 = 1 - beta1 ** state['step']
@@ -1825,8 +1831,9 @@ class AdaBelief(Optimizer):
 
 
                     if self.gradient_centralization in ['all', 'gc']:
-                        if len(list(G_grad.size())) > 1:
-                            G_grad.add_(-G_grad.mean(dim=tuple(range(1, len(list(G_grad.size())))), keepdim=True))
+                        if ndim(G_grad) > 1:
+                            G_grad = G_grad - G_grad.mean(axis=list(range(1, ndim(G_grad))), keepdims=True)
+
 
                     if any_abnormal_number(p_data_fp32):
                         sys.stderr.write('{0} p_data has abnormal value,trident automatically replace these abnormal value to zero.\n\r'.format(self.__class__.__name__))
@@ -1989,8 +1996,8 @@ class RangerAdaBelief(Optimizer):
                 # if grad.dim() > self.gc_gradient_threshold:
                 #    grad.add_(-grad.mean(dim=tuple(range(1, grad.dim())), keepdim=True))
                 if self.gradient_centralization in ['all', 'gcc']:
-                    if len(list(grad.size())) > 3:
-                        grad.add_(-grad.mean(dim=tuple(range(1, grad.dim())), keepdim=True))
+                    if ndim(grad)> 3:
+                        grad=grad-grad.mean(axis=list(range(1, ndim(grad))), keepdims=True)
 
                 state['step'] += 1
 
@@ -2042,8 +2049,8 @@ class RangerAdaBelief(Optimizer):
                 # GC operation
 
                 if self.gradient_centralization in ['all', 'gc']:
-                    if len(list(G_grad.size())) > 1:
-                        G_grad.add_(-G_grad.mean(dim=tuple(range(1, len(list(G_grad.size())))), keepdim=True))
+                    if ndim(G_grad)> 1:
+                        G_grad=G_grad-G_grad.mean(axis=list(range(1, ndim(G_grad))), keepdims=True)
 
 
                 p_data_fp32.add_(G_grad, alpha=-step_size * group['lr'])
@@ -2088,7 +2095,7 @@ class DiffGrad(Optimizer):
         https://openreview.net/forum?id=ryQu7f-RZ
     """
 
-    def __init__(self, params, lr=1e-3, betas=(0.9, 0.999), eps=1e-8, weight_decay=0, gradient_centralization=None):
+    def __init__(self, params, lr=1e-3, betas=(0.9, 0.999), eps=1e-7, weight_decay=0, gradient_centralization=None):
         if not 0.0 <= lr:
             raise ValueError("Invalid learning rate: {}".format(lr))
         if not 0.0 <= eps:
@@ -2149,8 +2156,9 @@ class DiffGrad(Optimizer):
 
 
                 if self.gradient_centralization in ['all', 'gcc']:
-                    if len(list(grad.size())) > 3:
-                        grad.add_(-grad.mean(dim=tuple(range(1, grad.dim())), keepdim=True))
+                    if ndim(grad)> 3:
+                        grad=grad-grad.mean(axis=list(range(1, ndim(grad))), keepdims=True)
+
                 state['step'] += 1
 
                 if group['weight_decay'] != 0:
@@ -2255,8 +2263,8 @@ class Lamb(Optimizer):
                 exp_avg, exp_avg_sq = state['exp_avg'], state['exp_avg_sq']
                 beta1, beta2 = group['betas']
                 if self.gradient_centralization in ['all', 'gcc']:
-                    if len(list(grad.size())) > 3:
-                        grad.add_(-grad.mean(dim=tuple(range(1, grad.dim())), keepdim=True))
+                    if ndim(grad)> 3:
+                        grad=grad-grad.mean(axis=list(range(1, ndim(grad))), keepdims=True)
 
                 state['step'] += 1
 
