@@ -9,6 +9,7 @@ import sys
 
 import numpy as np
 import tensorflow as tf
+from tensorflow.python.training.tracking import tracking
 from tensorflow.python.framework import ops
 from tensorflow.python.ops import array_ops
 from tensorflow.python.ops import math_ops
@@ -21,9 +22,10 @@ from trident.backend.common import camel2snake, get_class, epsilon, PrintExcepti
 from trident.backend.tensorspec import *
 from trident.backend.tensorflow_backend import *
 from trident.backend.tensorflow_ops import *
-from trident.optims.losses import Loss
-from trident.backend.common import dtype
+from trident.optims.losses import Loss,_check_logsoftmax_logit
 
+from trident.backend import dtype as Dtype
+from trident.backend.tensorspec import TensorSpec,TensorShape
 # def cosine_similarity(target, output):
 #     assert target.ndim == 2
 #     assert output.ndim == 2
@@ -44,7 +46,7 @@ __all__ = ['get_loss', '_ClassificationLoss', 'CrossEntropyLoss', 'BCELoss', 'MS
            'FocalLoss', 'DiceLoss', 'L1Loss', 'L2Loss', 'SmoothL1Loss', 'WingLoss', 'AdaptiveWingLoss', 'IoULoss', 'SoftIoULoss']
 
 
-class _ClassificationLoss(Loss):
+class _ClassificationLoss(Loss,tracking.AutoTrackable):
     """Calculate loss for  complex classification task."""
 
     def __init__(self, axis=-1, sample_weight=None, auto_balance=False, from_logits=False, ignore_index=-100, cutoff=None, label_smooth=False, reduction='mean', enable_ohem=False,
@@ -91,7 +93,9 @@ class _ClassificationLoss(Loss):
         self.ignore_index = ignore_index
         self.ignore_index_weight = None
         self.auto_balance = auto_balance
-        self.auto_balance = auto_balance
+        if self.auto_balance:
+            self.label_statistics = None
+            self._calculate_label_statistics()
 
         if cutoff is not None and not 0 < cutoff < 1:
             raise ValueError('cutoff should between 0 and 1')
@@ -104,17 +108,17 @@ class _ClassificationLoss(Loss):
     def _set_name_scope(self):
         """Creates a valid `name_scope` name."""
         if self.name is None:
-            name = self.__class__.__name__
+            self.name = self.__class__.__name__
         elif self.name == '<lambda>':
-            name = 'lambda'
+            self.name = 'lambda'
         else:
             # E.g. '_my_loss' => 'my_loss'
-            name = self.name.strip('_')
-        with ops.name_scope_v2(name) as scope_name:
+            self.name = self.name.strip('_')
+        with ops.name_scope_v2(self.name) as scope_name:
             self._name_scope = ops.name_scope_v2(scope_name)
 
     def _get_reduction(self, loss):
-        with self._name_scope:
+        with ops.name_scope(self.name+'_reduction', "reduction_loss", [loss]) as name:
             num_present = math_ops.cast(array_ops.size(loss, name='num_elements'), dtype=loss.dtype)
             if ndim(loss) == 0 or self.reduction == 'none':
                 return loss
@@ -125,8 +129,7 @@ class _ClassificationLoss(Loss):
                 loss = reshape(loss, (int_shape(loss)[0], -1))
                 return loss.mean(1).mean()
             elif self.reduction in ['mean', 'batch_mean']:
-                total_loss = math_ops.reduce_sum(loss)
-                return math_ops.div_no_nan(total_loss, num_present, name='value')
+                return math_ops.reduce_mean(loss ,name='value')
             elif self.reduction in ['sum', 'batch_sum']:
                 return math_ops.reduce_sum(loss)
             else:
@@ -135,17 +138,22 @@ class _ClassificationLoss(Loss):
 
     def flatten_check(self, output, target):
         "Check that `out` and `targ` have the same number of elements and flatten them."
-        if ndim(output) > 2 and ndim(output) == ndim(target) + 1:
-            shp = int_shape(output)
-            output = output.reshape((shp[0], -1, shp[-1]))
-            target = cast(target.reshape((shp[0], -1)), 'int64')
-            return output, target
-        elif ndim(output) > 2 and ndim(output) == ndim(target):
-            shp = int_shape(output)
-            output = output.reshape((shp[0], -1, shp[-1]))
-            if ndim(target) > 2:
-                target = target.reshape((shp[0], -1, shp[-1]))
-            return output, target
+        ndim_output = ndim(output)
+        ndim_target = ndim(target)
+        if ndim(output) > 2:
+            if self.axis == -1:
+                shp = int_shape(output)
+                output =output.reshape((shp[0], -1, shp[-1]))
+
+                if ndim_target == ndim_output - 1 and target.dtype == Dtype.long:
+                    target =cast(target.reshape((shp[0], -1)), 'int64')
+
+                elif ndim_target == ndim_output and target.dtype != Dtype.long:
+                        target = target.reshape((shp[0], -1, shp[-1]))
+
+                return output, target
+
+
         elif ndim(output) <= 2 and ndim(output) == ndim(target):
             return output, target
         elif ndim(output) <= 2 and ndim(output) == ndim(target) + 1:
@@ -172,7 +180,7 @@ class _ClassificationLoss(Loss):
 
             elif ds is not None and ds.__class__.__name__ == 'MaskDataset' and dp.traindata.label.object_type in [ObjectType.label_mask, ObjectType.color_mask]:
                 print('Start retrive label class distribution for auto-balance in loss function.')
-                unique, counts = tf.unique(to_tensor(np.stack([dp.traindata.label[i] for i in tqdm(range(len(dp.traindata.label)))]), dtype=dtype.long, device='cpu'),
+                unique, counts = tf.unique(to_tensor(np.stack([dp.traindata.label[i] for i in tqdm(range(len(dp.traindata.label)))]), dtype=Dtype.long, device='cpu'),
                                               return_counts=True)
                 unique = to_list(to_numpy(unique))
                 counts = to_numpy(counts)
@@ -217,28 +225,20 @@ class _ClassificationLoss(Loss):
 
         if (ndim(output) >= 1 and 'float' in str(output.dtype) and reduce_min(output) >= 0):
             self.is_logsoftmax = False
-
-            if reduce_max(output) > 1:
-                output = output / reduce_max(output)
-
             sum_output = reduce_sum(output, axis=self.axis, keepdims=True)
-            if reduce_max(abs(sum_output - ones_like(sum_output))) < 1e-3:
+            if reduce_max(output) > 1:
+                self.from_logits = False
+            elif reduce_max(abs(sum_output - ones_like(sum_output))) < 1e-3:
                 self.from_logits = True
             else:
-                output = output / clip(sum_output, 1e-7, 1 - 1e-7)
-                self.from_logits = True
+                #output = output / clip(sum_output, 1e-7, 1 - 1e-7)
+                self.from_logits = False
 
-            if self.auto_balance and self.label_statistics is not None:
-                output = output * to_tensor(self.label_statistics.copy())
-            output = clip(output, min=1e-8, max=1 - 1e-8)
 
-        elif (ndim(output) >= 1 and 'float' in str(output.dtype) and  reduce_max(output) <=0):
+
+        elif _check_logsoftmax_logit(output):
             self.is_logsoftmax = True
             self.from_logits = True
-            output = clip(output,min=-7.0,max=-1e-8)
-            if self.auto_balance and self.label_statistics is not None:
-                output = output + to_tensor(np.log(self.label_statistics.copy()))
-
         else:
             self.is_logsoftmax = False
             self.from_logits = False
@@ -265,14 +265,14 @@ class _ClassificationLoss(Loss):
             self.is_target_onehot = True
 
         # need target onehot but currently not
-        if target.dtype == tf.int64 and self.need_target_onehot == True and self.is_target_onehot == False:
+        if target.dtype == Dtype.int64 and self.need_target_onehot == True and self.is_target_onehot == False:
             target = make_onehot(target, num_classes=self.num_classes, axis=self.axis)
             self.is_target_onehot = True
             if self.label_smooth:
                 target = target + random_normal_like(target)
         return output, target
 
-    def calculate_loss(self, output, target, **kwargs):
+    def calculate_loss(self, output, target):
         """ Calculate the unaggregate loss.
         The loss function calculation logic should define here., please dont't aggregate the loss in this phase.
 
@@ -307,15 +307,35 @@ class _ClassificationLoss(Loss):
                 self._calculate_label_statistics()
             loss = self.calculate_loss(*self.preprocess(output, target.detach(), **kwargs))
             loss = self._handel_abnormal(loss)
-            loss = self._get_reduction(loss)
-            return loss
+            return self._get_reduction(loss)
         except Exception as e:
             print(e)
             PrintException()
             raise e
 
+    def _do_ohem(self, output: Tensor, target: Tensor):
+        if self.enable_ohem:
+            output_ = output.clone()
+            target_ = target.clone()
+            num_hard = 0
+            num_easy = 0
+            is_hard = None
+            if target.dtype == Dtype.int64:
+                is_hard = greater(target, 0)
+                num_hard =builtins.max( is_hard.sum().item(),1)
+                num_easy = int(self.ohem_ratio * num_hard)
+            hard_cases = is_hard > 0
 
-class _PairwiseLoss(Loss):
+            base_losses = nn.functional.nll_loss(output_, target_)
+            _, easy_cases = topk(base_losses * (1 - is_hard), num_easy)
+            idxs = easy_cases or hard_cases
+
+            output_hn = output.index_select(0, idxs)
+            target_hn = target.index_select(0, idxs)
+            return output_hn, target_hn
+
+
+class _PairwiseLoss(Loss,tracking.AutoTrackable):
     """Calculate loss for  complex classification task."""
 
     def __init__(self, axis=-1, reduction='batch_mean', enable_ohem=False, ohem_ratio=3.5, name=None, **kwargs):
@@ -359,31 +379,30 @@ class _PairwiseLoss(Loss):
     def _set_name_scope(self):
         """Creates a valid `name_scope` name."""
         if self.name is None:
-            name = self.__class__.__name__
+            self.name = self.__class__.__name__
         elif self.name == '<lambda>':
-            name = 'lambda'
+            self.name = 'lambda'
         else:
             # E.g. '_my_loss' => 'my_loss'
-            name = self.name.strip('_')
-        with ops.name_scope_v2(name) as scope_name:
+            self.name = self.name.strip('_')
+        with ops.name_scope_v2(self.name) as scope_name:
             self._name_scope = ops.name_scope_v2(scope_name)
 
     def _get_reduction(self, loss):
-        with self._name_scope:
+        with ops.name_scope(self.name+'_reduction', "reduction_loss", [loss]) as name:
             if ndim(loss) == 0 or self.reduction == 'none':
                 return loss
             num_present = math_ops.cast(array_ops.size(loss, name='num_elements'), dtype=loss.dtype)
             batch_size = math_ops.cast(tf.constant(array_ops.shape(loss, name='shape')[0]), dtype=loss.dtype)
 
             if ndim(loss) >= 2 and self.reduction == 'batch_sum':
-                total_loss = math_ops.div_no_nan(math_ops.reduce_sum(loss), batch_size, name='value')
+                loss = math_ops.div_no_nan(math_ops.reduce_sum(loss,0), batch_size, name='value')
                 return loss.mean(1).sum()
             elif ndim(loss) >= 2 and self.reduction == 'batch_mean':
-                total_loss = math_ops.reduce_sum(loss)
+                total_loss = math_ops.reduce_sum(loss,0)
                 return math_ops.div_no_nan(total_loss, math_ops.div_no_nan(num_present, batch_size), name='value')
             elif self.reduction in ('mean', 'batch_mean'):
-                total_loss = math_ops.reduce_sum(loss)
-                return math_ops.div_no_nan(total_loss, num_present, name='value')
+                return lmath_ops.reduce_mean(loss)
             elif self.reduction == ('sum', 'batch_sum'):
                 return math_ops.reduce_sum(loss)
             else:
@@ -422,12 +441,12 @@ class _PairwiseLoss(Loss):
         if output.shape == target.shape:
             return output, target
 
-        elif target.dtype == tf.int64 and ndim(output) == ndim(target) + 1:
+        elif target.dtype == Dtype.int64 and ndim(output) == ndim(target) + 1:
             num_class = int_shape(output)[self.axis]
             target = make_onehot(target, num_class, self.axis).float()
         return output, target
 
-    def calculate_loss(self, output, target, **kwargs):
+    def calculate_loss(self, output, target):
         """ Calculate the unaggregate loss.
         The loss function calculation logic should define here., please dont't aggregate the loss in this phase.
 
@@ -447,7 +466,7 @@ class _PairwiseLoss(Loss):
             hard_mask = None
             reduce_axis = list(range(output_.ndim))[1:]
             base_losses = pow(output_ - target, 2).mean(axis=reduce_axis) if len(reduce_axis) > 0 else pow(output_ - target, 2)
-            if target.dtype == dtype.int64:
+            if target.dtype == Dtype.int64:
                 hard_mask = target < 0
                 num_hard = reduce_sum(hard_mask).numpy()
                 num_easy = int(self.ohem_ratio * num_hard)
@@ -487,8 +506,7 @@ class _PairwiseLoss(Loss):
         try:
             output, target = self.flatten_check(output, target)
             loss = self.calculate_loss(*self.preprocess(output, target, **kwargs))
-            loss = self._get_reduction(loss)
-            return loss
+            return self._get_reduction(loss)
         except Exception as e:
             print(e)
             PrintException()
@@ -534,8 +552,41 @@ class CrossEntropyLoss(_ClassificationLoss):
         self._built = True
         self.need_target_onehot = False
 
-    def calculate_loss(self, output, target, **kwargs):
+    @tf.function
+    def calculate_loss(self, output, target):
         """
+        The usual cross-entropy cost is defined as:
+
+              labels * -log(sigmoid(logits)) +
+                  (1 - labels) * -log(1 - sigmoid(logits))
+
+          A value `pos_weight > 1` decreases the false negative count, hence increasing
+          the recall.
+          Conversely setting `pos_weight < 1` decreases the false positive count and
+          increases the precision.
+          This can be seen from the fact that `pos_weight` is introduced as a
+          multiplicative coefficient for the positive labels term
+          in the loss expression:
+
+              labels * -log(sigmoid(logits)) * pos_weight +
+                  (1 - labels) * -log(1 - sigmoid(logits))
+
+          For brevity, let `x = logits`, `z = labels`, `q = pos_weight`.
+          The loss is:
+
+                qz * -log(sigmoid(x)) + (1 - z) * -log(1 - sigmoid(x))
+              = qz * -log(1 / (1 + exp(-x))) + (1 - z) * -log(exp(-x) / (1 + exp(-x)))
+              = qz * log(1 + exp(-x)) + (1 - z) * (-log(exp(-x)) + log(1 + exp(-x)))
+              = qz * log(1 + exp(-x)) + (1 - z) * (x + log(1 + exp(-x))
+              = (1 - z) * x + (qz +  1 - z) * log(1 + exp(-x))
+              = (1 - z) * x + (1 + (q - 1) * z) * log(1 + exp(-x))
+
+          Setting `l = (1 + (q - 1) * z)`, to ensure stability and avoid overflow,
+          the implementation uses
+
+              (1 - z) * x + l * (log(1 + exp(-abs(x))) + max(-x, 0))
+
+          `logits` and `labels` must have the same type and shape.
 
         Args:
             output ():
@@ -544,60 +595,52 @@ class CrossEntropyLoss(_ClassificationLoss):
 
         Returns:
 
+
+        Examples:
+        >>> output = tf.constant(value=[[0.3, 0.3, 0.2], [0, 1, 0.5], [1, 1, 0]], dtype=tf.float32, shape=[3, 3])
+        >>> target = to_tensor([[0, 1, 0], [0, 1, 0], [1, 0, 0]],dtype=tf.float32)
+        >>> CrossEntropyLoss(reduction='mean')(output,target)
+        <tf.Tensor: shape=(), dtype=float32, numpy=0.8695473>
+        >>> CrossEntropyLoss(reduction='mean')(output,argmax(target,-1))
+        <tf.Tensor: shape=(), dtype=float32, numpy=0.8695473>
+        >>> CrossEntropyLoss(reduction='mean')(log_softmax(output),target)
+        <tf.Tensor: shape=(), dtype=float32, numpy=0.8695473>
+        >>> CrossEntropyLoss(reduction='mean')(log_softmax(output),argmax(target,-1))
+        <tf.Tensor: shape=(), dtype=float32, numpy=0.8695473>
+        >>> CrossEntropyLoss(reduction='mean', sample_weight=to_tensor([1,2,3]))(output,target)
+        <tf.Tensor: shape=(), dtype=float32, numpy=1.451763>
+
         """
-        # if self.is_logsoftmax == False:
-        #     if not self.from_logits:
-        #         output=softmax(output,self.axis)
-        #     loss =tf.nn.sparse_softmax_cross_entropy_with_logits(target,output,self.sample_weight,name='weighted_cross_entropy')
-        # else:
-        #     loss= -cast(target,output.dtype)*output*self.sample_weight
-        # return loss
-        # if self.sample_weight is not None and ndim(self.sample_weight)==1:
-        #     self.sample_weight=expand_dims(self.sample_weight,0)
-        # if self.ignore_index_weight is not None and ndim(self.ignore_index_weight)==1:
-        #     self.ignore_index_weight=expand_dims(self.ignore_index_weight,0)
-        with self._name_scope:
-            sample_weight = expand_dims(cast(self.sample_weight, output.dtype) * cast(self.ignore_index_weight, output.dtype), 0)
-            n_ = ndim(output) - ndim(sample_weight)
+        with ops.name_scope(self.name, "cross_entropy_loss", [output, target]) as name:
+            output = ops.convert_to_tensor(output, name="output")
+            target = array_ops.stop_gradient(ops.convert_to_tensor(target, name="target"))
+
+            sample_weight = array_ops.stop_gradient(cast(self.sample_weight, output.dtype) * cast(self.ignore_index_weight, output.dtype), name="sample_weight")
+
+            n_ = ndim(target) - ndim(sample_weight)
             for n in range(n_):
-                sample_weight = expand_dims(sample_weight, 1)
-            loss = None
+                sample_weight = expand_dims(sample_weight, 0)
+
             if self.is_target_onehot:
-                # -sum([p[i] * log2(q[i]) for i in range(len(p))])
                 if not self.is_logsoftmax:
-                    loss = nn.sparse_softmax_cross_entropy_with_logits_v2(labels=argmax(target, -1), logits=output)
-                    # loss = -reduce_sum(target * log_softmax(output, axis=self.axis, keepdims=True) * sample_weight, axis=-self.axis)
+                    output=nn.log_softmax_v2(output)
 
-                else:
+                    #return nn.softmax_cross_entropy_with_logits_v2(labels=target, logits=output,axis=-1)*sample_weight
+                return  -math_ops.reduce_sum(target * output*sample_weight, axis=1)
 
-                    # loss = nn.softmax_cross_entropy_with_logits_v2(labels=target, logits=exp(output))
-                    loss = -reduce_sum(target * output * sample_weight, axis=self.axis)
-
-                if ndim(loss) > 1:
-                    reduce_axes = list(range(loss.ndim))
-                    reduce_axes.remove(0)
-                    if len(reduce_axes) == 0:
-                        reduce_axes = None
-                    return reduce_mean(loss, axis=reduce_axes)
-                else:
-                    return loss
             elif not self.is_target_onehot and ndim(target) == ndim(output) - 1:
-                if self.is_logsoftmax:
-                    output = exp(output)
-                target = cast(target, cast_dtype=dtype.int64)
+                gather_sample_weight = tf.gather(sample_weight, target)
+                if not self.is_logsoftmax:
+                    output=nn.log_softmax_v2(output)
+
+                target = cast(target, cast_dtype=Dtype.int64)
+                gather_prob=tf.gather(output, target, batch_dims=1,axis=-1)
+
+
+                return math_ops.negative(gather_prob*gather_sample_weight)
+
 
-                loss = nn.sparse_softmax_cross_entropy_with_logits(labels=target, logits=output)
-                return loss
 
-                # loss = reduce_sum(loss, self.axis)
-                # if ndim(loss)>1:
-                #     reduce_axes = list(range(ndim(loss)))
-                #     reduce_axes.remove(0)
-                #     if len(reduce_axes) == 0:
-                #         reduce_axes = None
-                #     return reduce_mean(loss,axis=reduce_axes)
-                # else:
-                #     return loss
 
 
 class NLLLoss(_ClassificationLoss):
@@ -633,7 +676,7 @@ class NLLLoss(_ClassificationLoss):
         self._built = True
         self.need_target_onehot = True
 
-    def calculate_loss(self, output, target, **kwargs):
+    def calculate_loss(self, output, target):
         """
 
         Args:
@@ -695,23 +738,42 @@ class F1ScoreLoss(_ClassificationLoss):
         self.need_target_onehot = True
         self._built = True
 
-    def calculate_loss(self, output, target, **kwargs):
-        with self._name_scope:
+    @tf.function
+    def calculate_loss(self, output, target):
+        with ops.name_scope(self.name, "f1_score_loss", [output, target]) as name:
+            output = ops.convert_to_tensor(output, name="output")
+            target = array_ops.stop_gradient(ops.convert_to_tensor(target, name="target"))
+
+            sample_weight = array_ops.stop_gradient(
+                cast(self.sample_weight, output.dtype) * cast(self.ignore_index_weight, output.dtype),
+                name="sample_weight")
+            n_ = ndim(target) - ndim(sample_weight)
+            for n in range(n_):
+                sample_weight = expand_dims(sample_weight, 0)
+
+
             if self.is_logsoftmax:
-                output = clip(exp(output), 1e-8, 1 - 1e-8)
+                output = clip(exp(clip(output,max=0)),1e-7,1-1e-7)
                 self.from_logits = True
             if not self.from_logits:
                 output = softmax(output, self.axis)
-            if target.dtype == tf.int64 or self.is_target_onehot == False:
+            if target.dtype == Dtype.int64 or self.is_target_onehot == False:
                 target = cast(make_onehot(target, self.num_classes, axis=1), output.dtype)
 
-            tp = (target * output * self.sample_weight * self.ignore_index_weight).sum(axis=self.axis)
+            tp = (target * output)
             # tn = ((1 - target) * (1 - output))
             # fp = ((1 - target) * output)
             # fn = (target * (1 - output))
-            precision = true_divide(tp, reduce_sum(output, axis=self.axis))
-            recall = true_divide(tp, reduce_sum(target, axis=self.axis))
-            return 1 - (1 + self.beta ** 2) * precision * recall / (self.beta ** 2 * precision + recall)
+            #
+            #
+
+            # percision与recall，这里的K.epsilon代表一个小正数，用来避免分母为零
+            precision = tp / (output + epsilon())
+            recall= tp / (target + epsilon())
+
+            # 计算f1
+            return 1 - (1 + self.beta ** 2) * (precision * recall*sample_weight).sum(-1) / (self.beta ** 2 * precision + recall+epsilon()).sum(-1)
+
 
 
 class FocalLoss(_ClassificationLoss):
@@ -736,7 +798,8 @@ class FocalLoss(_ClassificationLoss):
         self.normalized = normalized
         self.need_target_onehot = True
 
-    def calculate_loss(self, output, target, **kwargs):
+    @tf.function
+    def calculate_loss(self, output, target):
         """
 
 
@@ -748,7 +811,14 @@ class FocalLoss(_ClassificationLoss):
         Returns:
 
             """
-        with self._name_scope:
+        with ops.name_scope(self.name, "focal_loss", [output, target]) as name:
+            output = ops.convert_to_tensor(output, name="output")
+            target = array_ops.stop_gradient(ops.convert_to_tensor(target, name="target"))
+
+            sample_weight = array_ops.stop_gradient(
+                cast(self.sample_weight, output.dtype) * cast(self.ignore_index_weight, output.dtype),
+                name="sample_weight")
+
             logp = CrossEntropyLoss()(output, target)
             p = exp(-logp)
 
@@ -781,7 +851,7 @@ class BCELoss(_ClassificationLoss):
         self.need_target_onehot = True
         self.is_target_onehot = False
 
-    def calculate_loss(self, output, target, **kwargs):
+    def calculate_loss(self, output, target):
         """
 
         Args:
@@ -792,9 +862,16 @@ class BCELoss(_ClassificationLoss):
         Returns:
 
         """
-        with self._name_scope:
+        with ops.name_scope(self.name, "bce_loss", [output, target]) as name:
+            output = ops.convert_to_tensor(output, name="output")
+            target = array_ops.stop_gradient(ops.convert_to_tensor(target, name="target"))
+
+            sample_weight = array_ops.stop_gradient(
+                cast(self.sample_weight, output.dtype) * cast(self.ignore_index_weight, output.dtype),
+                name="sample_weight")
+
             if self.is_logsoftmax:
-                output = exp(output)
+                output = clip(exp(clip(output,max=0)),1e-7,1-1e-7)
             loss = binary_cross_entropy(output, target, from_logits=self.from_logits)
             return loss
 
@@ -861,7 +938,7 @@ class DiceLoss(_ClassificationLoss):
         self.is_multiselection = False
         self._built = True
 
-    def calculate_loss(self, output, target, **kwargs):
+    def calculate_loss(self, output, target):
         """
 
         Args:
@@ -872,9 +949,16 @@ class DiceLoss(_ClassificationLoss):
         Returns:
 
         """
-        with self._name_scope:
+        with ops.name_scope(self.name, "dice_loss", [output, target]) as name:
+            output = ops.convert_to_tensor(output, name="output")
+            target = array_ops.stop_gradient(ops.convert_to_tensor(target, name="target"))
+
+            sample_weight = array_ops.stop_gradient(
+                cast(self.sample_weight, output.dtype) * cast(self.ignore_index_weight, output.dtype),
+                name="sample_weight")
+
             if self.is_logsoftmax:
-                output = exp(output)
+                output =clip(exp(clip(output,max=0)),1e-7,1-1e-7)
             reduce_axes = list(range(target.ndim))
             axis = self.axis if self.axis >= 0 else target.ndim + self.axis
             reduce_axes.remove(0)
@@ -907,7 +991,7 @@ class L1Loss(_PairwiseLoss):
         self.name = name
         self.reduction = reduction
 
-    def calculate_loss(self, output, target, **kwargs):
+    def calculate_loss(self, output, target):
         """
 
         Args:
@@ -918,7 +1002,9 @@ class L1Loss(_PairwiseLoss):
         Returns:
 
         """
-        with self._name_scope:
+        with ops.name_scope(self.name, "l1_loss", [output, target]) as name:
+            output = ops.convert_to_tensor(output, name="output")
+            target = array_ops.stop_gradient(ops.convert_to_tensor(target,dtype=output.dtype, name="target"))
             return tf.math.abs(output - target, name='l1_loss')
 
 
@@ -935,7 +1021,7 @@ class L2Loss(_PairwiseLoss):
         self.name = name
         self.reduction = reduction
 
-    def calculate_loss(self, output, target, **kwargs):
+    def calculate_loss(self, output, target):
         """
 
         Args:
@@ -946,7 +1032,9 @@ class L2Loss(_PairwiseLoss):
         Returns:
 
         """
-        with self._name_scope:
+        with ops.name_scope(self.name, "l2_loss", [output, target]) as name:
+            output = ops.convert_to_tensor(output, name="output")
+            target = array_ops.stop_gradient(ops.convert_to_tensor(target,dtype=output.dtype, name="target"))
             return tf.nn.l2_loss(output - target, name='l2_loss')
 
 
@@ -963,7 +1051,7 @@ class L2Loss(_PairwiseLoss):
 #         self.reduction = reduction
 #         self.huber_delta = 0.5
 #
-#     def calculate_loss(self, output, target, **kwargs):
+#     def calculate_loss(self, output, target):
 #         """
 #
 #         Args:
@@ -989,7 +1077,7 @@ class MSELoss(_PairwiseLoss):
         super().__init__(axis, reduction, name)
         self._built = True
 
-    def calculate_loss(self, output, target, **kwargs):
+    def calculate_loss(self, output, target):
         """
 
         Args:
@@ -1003,7 +1091,9 @@ class MSELoss(_PairwiseLoss):
         # with self._name_scope:
 
         # num_present = tf.reduce_sum(math_ops.cast(array_ops.size(output, name='num_elements'), dtype=output.dtype),name='reduce_sum')
-        with self._name_scope:
+        with ops.name_scope(self.name, "mse_loss", [output, target]) as name:
+            output = ops.convert_to_tensor(output, name="output")
+            target = array_ops.stop_gradient(ops.convert_to_tensor(target,dtype=output.dtype, name="target"))
             return (output - target)**2
         # return math_ops.div_no_nan(tf.nn.l2_loss(output-target), math_ops.cast(tf.shape(output)[0], dtype=output.dtype), name='value')
 
@@ -1021,7 +1111,7 @@ class SmoothL1Loss(_PairwiseLoss):
         self.reduction = reduction
         self.huber_delta = 0.5
 
-    def calculate_loss(self, output, target, **kwargs):
+    def calculate_loss(self, output, target):
         """
 
         Args:
@@ -1032,8 +1122,9 @@ class SmoothL1Loss(_PairwiseLoss):
         Returns:
 
         """
-        with self._name_scope:
-            target = math_ops.cast(target, dtype=output.dtype)
+        with ops.name_scope(self.name, "smooth_l1_loss", [output, target]) as name:
+            output = ops.convert_to_tensor(output, name="output")
+            target = array_ops.stop_gradient(ops.convert_to_tensor(target,dtype=output.dtype, name="target"))
             diff = abs(target - output)
             less_than_one = cast(less(diff, 1.0), tf.float32)  # Bool to float32
             smooth_l1_loss = (less_than_one * 0.5 * diff ** 2) + (1.0 - less_than_one) * (diff - 0.5)  # 同上图公式
@@ -1048,7 +1139,7 @@ class WingLoss(_PairwiseLoss):
         self.omega = omega
         self.epsilon = epsilon
 
-    def calculate_loss(self, output, target, **kwargs):
+    def calculate_loss(self, output, target):
         """
 
         Args:
@@ -1063,7 +1154,7 @@ class WingLoss(_PairwiseLoss):
         c = self.omega * (1.0 - log(1.0 + self.omega / self.epsilon))
 
         losses = where(
-            greater(delta_y, self.omega, dtype=dtype.bool),
+            greater(delta_y, self.omega, dtype=Dtype.bool),
             self.omega * log(1.0 + delta_y / self.epsilon),
             delta_y - c
         )
@@ -1080,7 +1171,7 @@ class AdaptiveWingLoss(_PairwiseLoss):
         self.epsilon = epsilon
         self.alpha = alpha
 
-    def calculate_loss(self, output, target, **kwargs):
+    def calculate_loss(self, output, target):
         """
 
         Args:
@@ -1124,7 +1215,7 @@ class EdgeLoss(_PairwiseLoss):
         else:
             return None
 
-    def calculate_loss(self, output, target, **kwargs):
+    def calculate_loss(self, output, target):
         with self._name_scope:
             loss1 = tf.reduce_mean(tf.math.square(self.first_order(output, 1) - self.first_order(target, 1)))
             loss2 = tf.reduce_mean(tf.math.square(self.first_order(output, 2) - self.first_order(target, 2)))
@@ -1141,7 +1232,7 @@ class IoULoss(_ClassificationLoss):
         self.is_logsoftmax = False
         self._built = True
 
-    def calculate_loss(self, output, target, **kwargs):
+    def calculate_loss(self, output, target):
         with self._name_scope:
             reshape_shape = [1] * ndim(output)
             reshape_shape[self.axis] = self.num_classes
@@ -1155,8 +1246,8 @@ class IoULoss(_ClassificationLoss):
                 output = softmax(output, self.axis)
 
             intersection = reduce_sum(output * target * sample_weight)
-            union = reduce_sum((output + target) * sample_weight) - intersection
-            loss = -log(intersection / union)
+            union = reduce_sum(output + target) - intersection
+            loss = -log(intersection / union+ 1e-7)
             return loss
 
 
@@ -1167,7 +1258,7 @@ class SoftIoULoss(Loss):
         self.reduction = reduction
         self.reduced = reduced
 
-    def calculate_loss(self, output, target, **kwargs):
+    def calculate_loss(self, output, target):
         # logit => N x Classes x H x W
         # target => N x H x W
 
