@@ -1,9 +1,11 @@
 import threading
 from multiprocessing import Pipe
 import numpy as np
+import subprocess as sp
+import torch.multiprocessing as mp
+from trident import context
 
-
-
+ctx=context._context()
 
 def worker(remote, parent_remote, env_fn):
     parent_remote.close()
@@ -86,4 +88,30 @@ class SubprocVecEnv():
             remote.send(('close', None))
         for p in self.ps:
             p.join()
-        self.closed = True
\ No newline at end of file
+        self.closed = True
+
+
+class MultipleEnvironments:
+    def __init__(self,*envs,num_envs,**kwargs):
+        if num_envs is None:
+            num_envs=len(envs)
+        self.agent_conns, self.env_conns = zip(*[mp.Pipe() for _ in range(num_envs)])
+
+        self.envs = list(envs)
+        self.num_states = self.envs[0].observation_space.shape[0]
+
+        for index in range(num_envs):
+            process = mp.Process(target=self.run, args=(index,))
+            process.start()
+            self.env_conns[index].close()
+
+    def run(self, index):
+        self.agent_conns[index].close()
+        while True:
+            request, action = self.env_conns[index].recv()
+            if request == "step":
+                self.env_conns[index].send(self.envs[index].step(action.item()))
+            elif request == "reset":
+                self.env_conns[index].send(self.envs[index].reset())
+            else:
+                raise NotImplementedError
\ No newline at end of file
